@@ -456,6 +456,12 @@ typedef LCMSHANDLE cmsHTRANSFORM;
 
 // HiFi separations, Thanks to Steven Greaves for providing the code,
 // the colorspace is not checked
+#define TYPE_CMYK5_8           (CHANNELS_SH(5)|BYTES_SH(1))
+#define TYPE_CMYK5_16          (CHANNELS_SH(5)|BYTES_SH(2))
+#define TYPE_CMYK5_16_SE       (CHANNELS_SH(5)|BYTES_SH(2)|ENDIAN16_SH(1))
+#define TYPE_KYMC5_8           (CHANNELS_SH(5)|BYTES_SH(1)|DOSWAP_SH(1))
+#define TYPE_KYMC5_16          (CHANNELS_SH(5)|BYTES_SH(2)|DOSWAP_SH(1))
+#define TYPE_KYMC5_16_SE       (CHANNELS_SH(5)|BYTES_SH(2)|DOSWAP_SH(1)|ENDIAN16_SH(1))
 
 #define TYPE_CMYKcm_8          (CHANNELS_SH(6)|BYTES_SH(1))
 #define TYPE_CMYKcm_8_PLANAR   (CHANNELS_SH(6)|BYTES_SH(1)|PLANAR_SH(1))
