@@ -40,10 +40,10 @@ typedef struct {
                } cmsJCh, FAR* LPcmsJCh;
 
 
-#define AVG_SURROUND_4	   0
-#define AVG_SURROUND	   1
-#define DIM_SURROUND	   2
-#define DARK_SURROUND	   3
+#define AVG_SURROUND_4     0
+#define AVG_SURROUND       1
+#define DIM_SURROUND       2
+#define DARK_SURROUND      3
 #define CUTSHEET_SURROUND  4
 
 
@@ -51,7 +51,7 @@ typedef struct {
 
               cmsCIEXYZ whitePoint;
               double    Yb;
-              double	La;
+              double    La;
               int       surround;
               double    D_value;
 
@@ -59,10 +59,10 @@ typedef struct {
 
 
 
-LCMSAPI HANDLE LCMSEXPORT cmsCIECAM97sInit(LPcmsViewingConditions pVC);
-LCMSAPI void   LCMSEXPORT cmsCIECAM97sDone(HANDLE hModel);
-LCMSAPI void   LCMSEXPORT cmsCIECAM97sForward(HANDLE hModel, LPcmsCIEXYZ pIn, LPcmsJCh pOut);
-LCMSAPI void   LCMSEXPORT cmsCIECAM97sReverse(HANDLE hModel, LPcmsJCh pIn,    LPcmsCIEXYZ pOut);
+LCMSAPI LCMSHANDLE LCMSEXPORT cmsCIECAM97sInit(LPcmsViewingConditions pVC);
+LCMSAPI void   LCMSEXPORT cmsCIECAM97sDone(LCMSHANDLE hModel);
+LCMSAPI void   LCMSEXPORT cmsCIECAM97sForward(LCMSHANDLE hModel, LPcmsCIEXYZ pIn, LPcmsJCh pOut);
+LCMSAPI void   LCMSEXPORT cmsCIECAM97sReverse(LCMSHANDLE hModel, LPcmsJCh pIn,    LPcmsCIEXYZ pOut);
 
 */
 
@@ -149,7 +149,7 @@ typedef struct {
 
 // Free model structure
 
-LCMSAPI void LCMSEXPORT cmsCIECAM97sDone(HANDLE hModel)
+LCMSAPI void LCMSEXPORT cmsCIECAM97sDone(LCMSHANDLE hModel)
 {
     LPcmsCIECAM97s lpMod = (LPcmsCIECAM97s) (LPSTR) hModel;
     if (lpMod) free(lpMod);
@@ -160,7 +160,7 @@ LCMSAPI void LCMSEXPORT cmsCIECAM97sDone(HANDLE hModel)
 static
 double discount(double d, double chan)
 {
-	return (d * chan + 1 - d);
+    return (d * chan + 1 - d);
 }
 
 
@@ -304,13 +304,13 @@ void ComputeHueQuadrature(double h, double* H, double* e)
 
 
 
-LCMSAPI HANDLE LCMSEXPORT cmsCIECAM97sInit(LPcmsViewingConditions pVC)
+LCMSAPI LCMSHANDLE LCMSEXPORT cmsCIECAM97sInit(LPcmsViewingConditions pVC)
 {
     LPcmsCIECAM97s lpMod;
     VEC3 tmp;
 
     if((lpMod = (LPcmsCIECAM97s) malloc(sizeof(cmsCIECAM97s))) == NULL) {
-		return (HANDLE) NULL;
+        return (LCMSHANDLE) NULL;
     }
 
 
@@ -363,13 +363,13 @@ LCMSAPI HANDLE LCMSEXPORT cmsCIECAM97sInit(LPcmsViewingConditions pVC)
 
 
     if (pVC->D_value == -1.0)
-   		  lpMod->calculate_D = 1;
+          lpMod->calculate_D = 1;
     else
     if (pVC->D_value == -2.0)
-		   lpMod->calculate_D = 2;
+           lpMod->calculate_D = 2;
     else {
-   		lpMod->calculate_D = 0;
-	   	lpMod->D = pVC->D_value;
+        lpMod->calculate_D = 0;
+        lpMod->D = pVC->D_value;
     }
 
    // Table I (revised)
@@ -377,41 +377,41 @@ LCMSAPI HANDLE LCMSEXPORT cmsCIECAM97sInit(LPcmsViewingConditions pVC)
    switch (lpMod->surround) {
 
     case AVG_SURROUND_4:
-	   lpMod->F = 1.0;
-	   lpMod->c = 0.69;
-	   lpMod->Fll = 0.0;    // Not included on Rev 2
-	   lpMod->Nc = 1.0;
-	   break;
+       lpMod->F = 1.0;
+       lpMod->c = 0.69;
+       lpMod->Fll = 0.0;    // Not included on Rev 2
+       lpMod->Nc = 1.0;
+       break;
     case AVG_SURROUND:
-	   lpMod->F = 1.0;
-	   lpMod->c = 0.69;
-	   lpMod->Fll = 1.0;
-	   lpMod->Nc = 1.0;
-	   break;
+       lpMod->F = 1.0;
+       lpMod->c = 0.69;
+       lpMod->Fll = 1.0;
+       lpMod->Nc = 1.0;
+       break;
     case DIM_SURROUND:
-	   lpMod->F = 0.99;
-	   lpMod->c = 0.59;
-	   lpMod->Fll = 1.0;
-	   lpMod->Nc = 0.95;
-	   break;
+       lpMod->F = 0.99;
+       lpMod->c = 0.59;
+       lpMod->Fll = 1.0;
+       lpMod->Nc = 0.95;
+       break;
     case DARK_SURROUND:
-	   lpMod->F = 0.9;
-	   lpMod->c = 0.525;
-	   lpMod->Fll = 1.0;
-	   lpMod->Nc = 0.8;
-	   break;
+       lpMod->F = 0.9;
+       lpMod->c = 0.525;
+       lpMod->Fll = 1.0;
+       lpMod->Nc = 0.8;
+       break;
     case CUTSHEET_SURROUND:
-	   lpMod->F = 0.9;
-	   lpMod->c = 0.41;
-	   lpMod->Fll = 1.0;
-	   lpMod->Nc = 0.8;
-	   break;
+       lpMod->F = 0.9;
+       lpMod->c = 0.41;
+       lpMod->Fll = 1.0;
+       lpMod->Nc = 0.8;
+       break;
     default:
-	   lpMod->F = 1.0;
-	   lpMod->c = 0.69;
-	   lpMod->Fll = 1.0;
-	   lpMod->Nc = 1.0;
-	   break;
+       lpMod->F = 1.0;
+       lpMod->c = 0.69;
+       lpMod->Fll = 1.0;
+       lpMod->Nc = 1.0;
+       break;
     }
 
     lpMod->k = 1 / (5 * lpMod->La  + 1);
@@ -419,9 +419,9 @@ LCMSAPI HANDLE LCMSEXPORT cmsCIECAM97sInit(LPcmsViewingConditions pVC)
 
     if (lpMod->calculate_D > 0) {
 
-	   lpMod->D = lpMod->F * (1 - 1 / (1 + 2*pow(lpMod->La, 0.25) + pow(lpMod->La, 2)/300.0));
-	   if (lpMod->calculate_D > 1)
-		   lpMod->D = (lpMod->D + 1.0) / 2;
+       lpMod->D = lpMod->F * (1 - 1 / (1 + 2*pow(lpMod->La, 0.25) + pow(lpMod->La, 2)/300.0));
+       if (lpMod->calculate_D > 1)
+           lpMod->D = (lpMod->D + 1.0) / 2;
     }
 
 
@@ -459,7 +459,7 @@ LCMSAPI HANDLE LCMSEXPORT cmsCIECAM97sInit(LPcmsViewingConditions pVC)
 
     lpMod->A_subw=lpMod->Nbb*(2.0*lpMod->RGB_subaw_prime.n[0]+lpMod->RGB_subaw_prime.n[1]+lpMod->RGB_subaw_prime.n[2]/20.0-NOISE_CONSTANT);
 
-    return (HANDLE) lpMod;
+    return (LCMSHANDLE) lpMod;
 }
 
 
@@ -469,7 +469,7 @@ LCMSAPI HANDLE LCMSEXPORT cmsCIECAM97sInit(LPcmsViewingConditions pVC)
 // The forward model: XYZ -> JCh
 //
 
-LCMSAPI void LCMSEXPORT cmsCIECAM97sForward(HANDLE hModel, LPcmsCIEXYZ inPtr, LPcmsJCh outPtr)
+LCMSAPI void LCMSEXPORT cmsCIECAM97sForward(LCMSHANDLE hModel, LPcmsCIEXYZ inPtr, LPcmsJCh outPtr)
 {
 
         LPcmsCIECAM97s lpMod = (LPcmsCIECAM97s) (LPSTR) hModel;
@@ -478,7 +478,7 @@ LCMSAPI void LCMSEXPORT cmsCIECAM97sForward(HANDLE hModel, LPcmsCIEXYZ inPtr, LP
 
         if (inPtr -> Y <= 0.0) {
 
-	  outPtr -> J = outPtr -> C = outPtr -> h = 0.0;
+      outPtr -> J = outPtr -> C = outPtr -> h = 0.0;
           return;
         }
 
@@ -563,7 +563,7 @@ LCMSAPI void LCMSEXPORT cmsCIECAM97sForward(HANDLE hModel, LPcmsCIEXYZ inPtr, LP
 //
 
 
-LCMSAPI void LCMSEXPORT cmsCIECAM97sReverse(HANDLE hModel, LPcmsJCh inPtr, LPcmsCIEXYZ outPtr)
+LCMSAPI void LCMSEXPORT cmsCIECAM97sReverse(LCMSHANDLE hModel, LPcmsJCh inPtr, LPcmsCIEXYZ outPtr)
 {
     LPcmsCIECAM97s lpMod = (LPcmsCIECAM97s) (LPSTR) hModel;
     double J, C, h, A, H1val, es, s, a, b;
@@ -606,8 +606,7 @@ LCMSAPI void LCMSEXPORT cmsCIECAM97sReverse(HANDLE hModel, LPcmsJCh inPtr, LPcms
 
 
     ComputeHueQuadrature(h, &H1val, &es);
-    // ComputeHueQuadrature(h, &H1val, &h1, &e1, &h2, &e2, &es);
-
+    
     // (7) Calculate s
 
     s = pow(C / (2.44 * pow(J/100.0, 0.67*lpMod->n) * (1.64 - pow(0.29, lpMod->n))) , (1./0.69));
@@ -620,10 +619,10 @@ LCMSAPI void LCMSEXPORT cmsCIECAM97sReverse(HANDLE hModel, LPcmsJCh inPtr, LPcms
     sec_h = sqrt(1 + tan_h * tan_h);
 
     if ((h > 90) && (h < 270))
-			sec_h = -sec_h;
+            sec_h = -sec_h;
 
     a = s * ( A/lpMod->Nbb + NOISE_CONSTANT) / ( sec_h * 50000.0 * es * lpMod->Nc * lpMod->Ncb/ 13.0 +
-		   s * (11.0 / 23.0 + (108.0/23.0) * tan_h));
+           s * (11.0 / 23.0 + (108.0/23.0) * tan_h));
 
     b = a * tan_h;
 
@@ -637,35 +636,35 @@ LCMSAPI void LCMSEXPORT cmsCIECAM97sReverse(HANDLE hModel, LPcmsJCh inPtr, LPcms
 
     if ((R_suba_prime - 1) < 0) {
 
-		 R_prime = -100.0 * pow((2.0 - 2.0 * R_suba_prime) /
-							(39.0 + R_suba_prime), 1.0/0.73);
+         R_prime = -100.0 * pow((2.0 - 2.0 * R_suba_prime) /
+                            (39.0 + R_suba_prime), 1.0/0.73);
     }
     else
     {
-		 R_prime = 100.0 * pow((2.0 * R_suba_prime - 2.0) /
-							(41.0 - R_suba_prime), 1.0/0.73);
+         R_prime = 100.0 * pow((2.0 * R_suba_prime - 2.0) /
+                            (41.0 - R_suba_prime), 1.0/0.73);
     }
 
     if ((G_suba_prime - 1) < 0)
     {
-		 G_prime = -100.0 * pow((2.0 - 2.0 * G_suba_prime) /
-							(39.0 + G_suba_prime), 1.0/0.73);
+         G_prime = -100.0 * pow((2.0 - 2.0 * G_suba_prime) /
+                            (39.0 + G_suba_prime), 1.0/0.73);
     }
     else
     {
-		 G_prime = 100.0 * pow((2.0 * G_suba_prime - 2.0) /
-							(41.0 - G_suba_prime), 1.0/0.73);
+         G_prime = 100.0 * pow((2.0 * G_suba_prime - 2.0) /
+                            (41.0 - G_suba_prime), 1.0/0.73);
     }
 
     if ((B_suba_prime - 1) < 0)
     {
-		 B_prime = -100.0 * pow((2.0 - 2.0 * B_suba_prime) /
-							(39.0 + B_suba_prime), 1.0/0.73);
+         B_prime = -100.0 * pow((2.0 - 2.0 * B_suba_prime) /
+                            (39.0 + B_suba_prime), 1.0/0.73);
     }
     else
     {
-		 B_prime = 100.0 * pow((2.0 * B_suba_prime - 2.0) /
-							(41.0 - B_suba_prime), 1.0/0.73);
+         B_prime = 100.0 * pow((2.0 * B_suba_prime - 2.0) /
+                            (41.0 - B_suba_prime), 1.0/0.73);
     }
 
 
@@ -681,7 +680,7 @@ LCMSAPI void LCMSEXPORT cmsCIECAM97sReverse(HANDLE hModel, LPcmsJCh inPtr, LPcms
 
 #ifdef USE_CIECAM97s2
 
-	   // (12)
+       // (12)
 
 
            RvAdaptationDegree(lpMod, &RGBY, &RGB_subc_Y);
@@ -695,7 +694,7 @@ LCMSAPI void LCMSEXPORT cmsCIECAM97sReverse(HANDLE hModel, LPcmsJCh inPtr, LPcms
 
            // (12) Calculate Yc
 
-	   Y_subc = 0.43231*RGB_subc_Y.n[0]+0.51836*RGB_subc_Y.n[1]+0.04929*RGB_subc_Y.n[2];
+       Y_subc = 0.43231*RGB_subc_Y.n[0]+0.51836*RGB_subc_Y.n[1]+0.04929*RGB_subc_Y.n[2];
 
            // (13) Calculate (Y/Yc)R, (Y/Yc)G and (Y/Yc)B
 
@@ -703,20 +702,19 @@ LCMSAPI void LCMSEXPORT cmsCIECAM97sReverse(HANDLE hModel, LPcmsJCh inPtr, LPcms
            RvAdaptationDegree(lpMod, &Y_over_Y_subc_RGB, &RGB_subc_Y);
 
            // (14) Calculate Y'
-	   Y_prime = 0.43231*(Y_over_Y_subc_RGB.n[0]*Y_subc) + 0.51836*(Y_over_Y_subc_RGB.n[1]*Y_subc) + 0.04929 * (Y_over_Y_subc_RGB.n[2]*Y_subc);
+       Y_prime = 0.43231*(Y_over_Y_subc_RGB.n[0]*Y_subc) + 0.51836*(Y_over_Y_subc_RGB.n[1]*Y_subc) + 0.04929 * (Y_over_Y_subc_RGB.n[2]*Y_subc);
 
            if (Y_prime < 0 || Y_subc < 0)
            {
                 // Discard to near black point
-                // cmsSignalError(-1, "CIECAM97s: bad Y'\n");
-
+               
                 outPtr -> X = 0;
                 outPtr -> Y = 0;
                 outPtr -> Z = 0;
                 return;
            }
 
-	   B_term = pow(Y_prime / Y_subc, (1.0 / lpMod->p) - 1);
+       B_term = pow(Y_prime / Y_subc, (1.0 / lpMod->p) - 1);
 
           // (15) Calculate X'', Y'' and Z''
            Y_over_Y_subc_RGB.n[2] /= B_term;
