@@ -29,7 +29,6 @@
 // Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 #include "lcms.h"
-#include <stdarg.h>
 
 
 // As a rule, only the functions visibles from API can signal
@@ -42,14 +41,20 @@ int  LCMSEXPORT cmsErrorAction(int lAbort);
 // ******************************************************************
 
 static int nDoAbort = LCMS_ERROR_ABORT;
+static cmsErrorHandlerFunction UserErrorHandler = (cmsErrorHandlerFunction) NULL;
 
 
 int LCMSEXPORT cmsErrorAction(int nAction)
 {
-       int nOld = nDoAbort;	   
-	   nDoAbort = nAction;
+       int nOld = nDoAbort;    
+       nDoAbort = nAction;
 
-	   return nOld;
+       return nOld;
+}
+
+void LCMSEXPORT cmsSetErrorHandler(cmsErrorHandlerFunction Fn)
+{
+    UserErrorHandler = Fn;
 }
 
 
@@ -60,10 +65,23 @@ void cmsSignalError(int ErrorCode, const char *ErrorText, ...)
 {
        va_list args;
 
-      
+       
        if (nDoAbort == LCMS_ERROR_IGNORE) return;
 
-		va_start(args, ErrorText);
+        va_start(args, ErrorText);
+
+        if (UserErrorHandler != NULL) {
+
+            char Buffer[1024];
+            
+            vsprintf(Buffer, ErrorText, args);
+            va_end(args);   
+
+            if (UserErrorHandler(ErrorCode, Buffer)) {     
+                   
+                return;
+            }
+       }
 
 #if defined( __CONSOLE__ ) || defined( NON_WINDOWS )
 
@@ -83,15 +101,15 @@ void cmsSignalError(int ErrorCode, const char *ErrorText, ...)
               strcat(Buffer1, Buffer2);
               MessageBox(NULL, Buffer1, "Little cms",
                                           MB_OK|MB_ICONSTOP|MB_TASKMODAL);
-			  va_end(args);
+              va_end(args);
 
               if (nDoAbort == LCMS_ERROR_ABORT) {
 
 #ifdef __BORLANDC__
-					_cexit();
+                    _cexit();
 #endif
 
-	              FatalAppExit(0, "lcms is terminating application");
+                  FatalAppExit(0, "lcms is terminating application");
               }
 
               }
