@@ -34,37 +34,36 @@
 
 LPGAMMATABLE LCMSEXPORT cmsAllocGamma(int nEntries);
 void         LCMSEXPORT cmsFreeGamma(LPGAMMATABLE Gamma);
-void		 LCMSEXPORT cmsFreeGammaTriple(LPGAMMATABLE Gamma[3]);
+void         LCMSEXPORT cmsFreeGammaTriple(LPGAMMATABLE Gamma[3]);
 LPGAMMATABLE LCMSEXPORT cmsBuildGamma(int nEntries, double Gamma);
+LPGAMMATABLE LCMSEXPORT cmsDupGamma(LPGAMMATABLE Src);
 LPGAMMATABLE LCMSEXPORT cmsReverseGamma(int nResultSamples, LPGAMMATABLE InGamma);
 LPGAMMATABLE LCMSEXPORT cmsJoinGamma(LPGAMMATABLE InGamma, LPGAMMATABLE OutGamma);
 LPGAMMATABLE LCMSEXPORT cmsJoinGammaEx(LPGAMMATABLE InGamma, LPGAMMATABLE OutGamma, int nPoints);
-BOOL		 LCMSEXPORT cmsSmoothGamma(LPGAMMATABLE Tab, double lambda);
+BOOL         LCMSEXPORT cmsSmoothGamma(LPGAMMATABLE Tab, double lambda);
 
-
-LPGAMMATABLE cdecl cmsScaleGamma(LPGAMMATABLE Input, Fixed32 Factor);
 BOOL         cdecl _cmsSmoothEndpoints(LPWORD Table, int nPoints);
 
 
-
-
 // Sampled curves
 
 LPSAMPLEDCURVE cdecl cmsAllocSampledCurve(int nItems);
 void           cdecl cmsFreeSampledCurve(LPSAMPLEDCURVE p);
-void		   cdecl cmsEndpointsOfSampledCurve(LPSAMPLEDCURVE p, double* Min, double* Max);
-void		   cdecl cmsClampSampledCurve(LPSAMPLEDCURVE p, double Min, double Max);
+void           cdecl cmsEndpointsOfSampledCurve(LPSAMPLEDCURVE p, double* Min, double* Max);
+void           cdecl cmsClampSampledCurve(LPSAMPLEDCURVE p, double Min, double Max);
 BOOL           cdecl cmsSmoothSampledCurve(LPSAMPLEDCURVE Tab, double SmoothingLambda);
-void		   cdecl cmsRescaleSampledCurve(LPSAMPLEDCURVE p, double Min, double Max, int nPoints);
+void           cdecl cmsRescaleSampledCurve(LPSAMPLEDCURVE p, double Min, double Max, int nPoints);
 
 LPSAMPLEDCURVE cdecl cmsJoinSampledCurves(LPSAMPLEDCURVE X, LPSAMPLEDCURVE Y, int nResultingPoints);
 
+double LCMSEXPORT cmsEstimateGamma(LPGAMMATABLE t);
+double LCMSEXPORT cmsEstimateGammaEx(LPWORD GammaTable, int nEntries, double Thereshold);
 
 // ----------------------------------------------------------------------------------------
 
 // #define DEBUG 1
 
-#define MAX_KNOTS	4096
+#define MAX_KNOTS   4096
 typedef float vec[MAX_KNOTS+1];
 
 
@@ -72,6 +71,8 @@ typedef float vec[MAX_KNOTS+1];
 
 #if 1
 
+// No slope limiting
+
 static
 double FGamma(double R, double x)
 {
@@ -82,33 +83,33 @@ double FGamma(double R, double x)
 
 // Quantization becomes evident below 
 //
-//		R^x * 65535 < 1.0
+//      R^x * 65535 < 1.0
 //
 // So, we apply linear tram on
 // 
-//		R^x < 1 / 65535
+//      R^x < 1 / 65535
 //
-//		x * log(R) < log(1) - log(65535) 
+//      x * log(R) < log(1) - log(65535) 
 //
-//		R < (1 /exp(log(65535) / x)
+//      R < (1 /exp(log(65535) / x)
 //
-//	Somehow arbitrarely, we adopt 0.018, which corresponds
+//  Somehow arbitrarely, we adopt 0.018, which corresponds
 //  with gamma 2.7 This is adequate for most situations.
 //
 //  The resulting line must be:
 //
-//	f(0) = 0
-//	f(0.018) = pow(0.018, x)
+//  f(0) = 0
+//  f(0.018) = pow(0.018, x)
 //
-//	so, 
-//	
-//	f() = pow(0.018, x) / 0.018
+//  so, 
+//  
+//  f() = pow(0.018, x) / 0.018
 
 static 
 double FGamma(double R, double x)
 {
-	   if (R < 0.018) 
-		   return (R * pow(0.018, x)) / 0.018;
+       if (R < 0.018) 
+           return (R * pow(0.018, x)) / 0.018;
 
        return pow(R, x);
 }
@@ -134,7 +135,7 @@ LPGAMMATABLE LCMSEXPORT cmsAllocGamma(int nEntries)
 
 void LCMSEXPORT cmsFreeGamma(LPGAMMATABLE Gamma)
 {
-       free(Gamma);
+       if (Gamma) free(Gamma);
 }
 
 
@@ -171,15 +172,15 @@ LPGAMMATABLE LCMSEXPORT cmsBuildGamma(int nEntries, double Gamma)
               return p;
        }
 
-	   if (Gamma == 1.0) {
+       if (Gamma == 1.0) {
 
-		   for (i=0; i < nEntries; i++) {
+           for (i=0; i < nEntries; i++) {
 
-			Table[i] = _cmsQuantizeVal(i, nEntries);              
+            Table[i] = _cmsQuantizeVal(i, nEntries);              
 
-		   }
-		   return p;
-	   }
+           }
+           return p;
+       }
 
        for (i=0; i < nEntries; i++)
        {
@@ -193,6 +194,23 @@ LPGAMMATABLE LCMSEXPORT cmsBuildGamma(int nEntries, double Gamma)
 }
 
 
+// Duplicate a gamma table
+
+LPGAMMATABLE  LCMSEXPORT cmsDupGamma(LPGAMMATABLE In)
+{
+       LPGAMMATABLE Ptr;
+
+       Ptr = cmsAllocGamma(In -> nEntries);
+       if (Ptr == NULL) return NULL;
+
+       CopyMemory(Ptr -> GammaTable,
+                  In -> GammaTable,
+                  In -> nEntries * sizeof(WORD));
+
+       return Ptr;
+}
+
+
 // Handle gamma using interpolation tables. The resulting curves can become
 // very stange, but are pleasent to eye.
 
@@ -231,53 +249,37 @@ LPGAMMATABLE LCMSEXPORT cmsJoinGamma(LPGAMMATABLE InGamma,
 // New method, using smoothed parametric curves. This works FAR better.
 // We want to get 
 //
-//		y = f(g^-1(x))		; f = ingamma, g = outgamma
+//      y = f(g^-1(x))      ; f = ingamma, g = outgamma
 //
 // And this can be parametrized as
 //
-//		y = f(t)
-//		x = g(t)
+//      y = f(t)
+//      x = g(t)
 
 
 LPGAMMATABLE LCMSEXPORT cmsJoinGammaEx(LPGAMMATABLE InGamma,
-									   LPGAMMATABLE OutGamma, int nPoints)
+                                       LPGAMMATABLE OutGamma, int nPoints)
 {
 
-	LPSAMPLEDCURVE x, y, r;
-	LPGAMMATABLE res;
-	
-	x = cmsConvertGammaToSampledCurve(InGamma,  nPoints);
-	y = cmsConvertGammaToSampledCurve(OutGamma, nPoints);
-	r = cmsJoinSampledCurves(y, x, nPoints);
+    LPSAMPLEDCURVE x, y, r;
+    LPGAMMATABLE res;
+    
+    x = cmsConvertGammaToSampledCurve(InGamma,  nPoints);
+    y = cmsConvertGammaToSampledCurve(OutGamma, nPoints);
+    r = cmsJoinSampledCurves(y, x, nPoints);
 
-	// Does clean "hair"
-	cmsSmoothSampledCurve(r, 0.001);
-	
-	cmsFreeSampledCurve(x);
-	cmsFreeSampledCurve(y);
+    // Does clean "hair"
+    cmsSmoothSampledCurve(r, 0.001);
+    
+    cmsFreeSampledCurve(x);
+    cmsFreeSampledCurve(y);
 
-	res = cmsConvertSampledCurveToGamma(r, 65535.0);
-	cmsFreeSampledCurve(r);
+    res = cmsConvertSampledCurveToGamma(r, 65535.0);
+    cmsFreeSampledCurve(r);
 
-	return res;
+    return res;
 }
 
-// This does reverse a curve using smooth parametric curves. 
-
-
-/*
-LPGAMMATABLE LCMSEXPORT cmsReverseGamma2(int nResultSamples, LPGAMMATABLE InGamma)
-{
-	LPGAMMATABLE Straight = cmsBuildGamma(InGamma->nEntries, 1.0);
-	LPGAMMATABLE Result;
-	
-	Result   = cmsJoinGammaEx(Straight, InGamma, nResultSamples);
-
-	cmsFreeGamma(Straight);
-
-	return Result;
-}
-*/
 
 
 // Reverse a gamma table
@@ -299,38 +301,25 @@ LPGAMMATABLE LCMSEXPORT cmsReverseGamma(int nResultSamples, LPGAMMATABLE InGamma
        {
               WORD wValIn, wValOut;
 
-			  wValIn = _cmsQuantizeVal(i, nResultSamples);           
+              wValIn = _cmsQuantizeVal(i, nResultSamples);           
               wValOut = cmsReverseLinearInterpLUT16(wValIn, InPtr, &L16In);
               p -> GammaTable[i] = wValOut;
        }
 
 
-	   // Does eliminate "hair" on curve	  
-	   cmsSmoothGamma(p, 0.001);	   	   
-	   	   
+       // Does eliminate "hair" on curve      
+       cmsSmoothGamma(p, 0.001);           
+           
        return p;
 }
 
 
 
-LPGAMMATABLE cmsScaleGamma(LPGAMMATABLE Input, Fixed32 Factor)
-{
-       LPGAMMATABLE Output;
-       int i, nEntries;
-
-
-       nEntries = Input -> nEntries;
-       Output = cmsAllocGamma(nEntries);
-       if (!Output) return NULL;
-
-       for (i=0; i < nEntries; i++)
-              Output->GammaTable[i] = FixedScale(Input->GammaTable[i], Factor);
-
-       return Output;
-}
-
-
+// Parametric curves
+//
 // Parameters goes as: Gamma, a, b, c, d, e, f
+// Type is the ICC type +1
+// if type is negative, then the curve is analyticaly inverted
 
 LPGAMMATABLE LCMSEXPORT cmsBuildParametricGamma(int nEntries, int Type, double Params[])
 {
@@ -349,89 +338,143 @@ LPGAMMATABLE LCMSEXPORT cmsBuildParametricGamma(int nEntries, int Type, double P
                 switch (Type) {
 
                 // X = Y ^ Gamma
-                case 0:
+                case 1:
                       Val = pow(R, Params[0]);
                       break;
 
+                // Type 1 Reversed: X = Y ^1/gamma
+                case -1:
+                      Val = pow(R, 1/Params[0]);
+                      break;
+
                 // CIE 122-1966
-                // Y = (aX + b)^Gamma + c | -b <= X <= a
-                // Y = 0                  | else
+                // Y = (aX + b)^Gamma  | X >= -b/a
+                // Y = 0               | else
+                case 2:
+                    if (R >= -Params[2] / Params[1]) {
+                              
+                              e = Params[1]*R + Params[2];
 
-                case 1:
-					if (R >= -Params[1] && R <= Params[2]) {
-							  
-							  e = Params[1]*R + Params[2];
-
-							  if (e > 0)
-								Val = pow(e, Params[0]) + Params[3];
-							  else
-								Val = Params[3];
-					}
+                              if (e > 0)
+                                Val = pow(e, Params[0]);
+                              else
+                                Val = 0;
+                    }
                     else
                               Val = 0;
                       break;
 
-                // IEC 61966-3
-                // Y = (aX + b)^Gamma + c | -b <= X <= a
-                // Y = c                  | else
+                // Type 2 Reversed
+                // X = (Y ^1/g  - b) / a
+                case -2: 
+                    
+                    Val = (pow(R, 1.0/Params[0]) - Params[2]) / Params[1];
+                    if (Val < 0)
+                            Val = 0;                            
+                    break;
 
-                case 2:
-					if (R >= -Params[1] && R <= Params[2]) {
-							
-							  e = Params[1]*R + Params[2];
-							  if (e > 0)
-								Val = pow(e, Params[0]) + Params[3];
-							  else
-								Val = Params[3];
-					}
+
+                // IEC 61966-3
+                // Y = (aX + b)^Gamma | X <= -b/a
+                // Y = c              | else
+                case 3:
+                    if (R >= -Params[2] / Params[1]) {
+                            
+                      e = Params[1]*R + Params[2];                    
+                      Val = pow(e, Params[0]) + Params[3];
+                    }
                     else
-                              Val = Params[3];
-                      break;
+                      Val = Params[3];
+                    break;
+
+
+                // Type 3 reversed
+                // X=((Y-c)^1/g - b)/a      | (Y>=c)
+                // X=-b/a                   | (Y<c) 
+                    
+                case -3:
+                    if (R >= Params[3])  {
+                        e = R - Params[3];
+                        Val = (pow(e, 1/Params[0]) - Params[2]) / Params[1];
+                        if (Val < 0) Val = 0;
+                    }
+                    else {
+                        Val = -Params[2] / Params[1];
+                    }
+                    break;
+
 
                 // IEC 61966-2.1 (sRGB)
                 // Y = (aX + b)^Gamma | X >= d
                 // Y = cX             | X < d
-
-                case 3:
-					if (R >= Params[4]) {
-							  
-							  e = Params[1]*R + Params[2];
-							  if (e > 0)
-								Val = pow(e, Params[0]);
-							  else
-								Val = 0;
-					}
+                case 4:
+                    if (R >= Params[4]) {
+                              
+                              e = Params[1]*R + Params[2];
+                              if (e > 0)
+                                Val = pow(e, Params[0]);
+                              else
+                                Val = 0;
+                    }
                       else
                               Val = R * Params[3];
                       break;
 
-                // Y = (aX + b)^Gamma + c | -b <= X <= a
-                // Y = 0                  | else
+                // Type 4 reversed
+                // X=((Y^1/g-b)/a)    | Y >= (ad+b)^g
+                // X=Y/c              | Y< (ad+b)^g
 
-                case 4:
-					if (R >= Params[4]) {
-							  e = Params[1]*R + Params[2];
-							  if (e > 0) 
-								Val = pow(e, Params[0]) + Params[3];
-							  else
-								Val = Params[3];
-					}
+                case -4:
+                    if (R >= pow(Params[1] * Params[4] + Params[2], Params[0])) {
+
+                        Val = (pow(R, 1.0/Params[0]) - Params[2]) / Params[1];
+                    }
+                    else {
+                        Val = R / Params[3];
+                    }
+                    break;
+                
+
+
+                // Y = (aX + b)^Gamma + e | X <= d
+                // Y = cX + f             | else
+                case 5:
+                    if (R >= Params[4]) {
+                             
+                        e = Params[1]*R + Params[2];
+                        Val = pow(e, Params[0]) + Params[5];
+                    }        
                     else
-                              Val = R*Params[5] + Params[6];
+                        Val = R*Params[3] + Params[6];
                     break;
 
+
+                // Reversed type 5
+                // X=((Y-e)1/g-b)/a   | Y >=(ad+b)^g+e)
+                // X=(Y-f)/c          | else
+                case -5:
+
+                if (R >= pow(Params[1] * Params[4], Params[0]) + Params[5]) {
+
+                    Val = pow(R - Params[5], 1/Params[0]) - Params[2] / Params[1];
+                }
+                else {
+                    Val = (R - Params[6]) / Params[3];
+                }
+                break;
+
                 default:
-                        cmsSignalError(-1, "Unsupported parametrized curve");
+                        cmsSignalError(LCMS_ERRC_ABORTED, "Unsupported parametric curve type=%d", abs(Type)-1);
                         cmsFreeGamma(Table);
                         return NULL;
                 }
 
 
-		// Saturate
+        // Saturate
 
-		dval = Val * 65535.0 + .5;
-		if (dval > 65535.) dval = 65535.0;
-		if (dval < 0) dval = 0;
+        dval = Val * 65535.0 + .5;
+        if (dval > 65535.) dval = 65535.0;
+        if (dval < 0) dval = 0;
 
         Table->GammaTable[i] = (WORD) floor(dval);
         }
@@ -441,8 +484,7 @@ LPGAMMATABLE LCMSEXPORT cmsBuildParametricGamma(int nEntries, int Type, double P
 
 
 // From: Eilers, P.H.C. (1994) Smoothing and interpolation with finite
-// differences. in: Graphic Gems IV, Heckbert, P.S. (ed.), Academic
-// press.
+// differences. in: Graphic Gems IV, Heckbert, P.S. (ed.), Academic press.
 //
 // Smoothing and interpolation with second differences.
 //
@@ -490,64 +532,64 @@ void smooth2(vec w, vec y, vec z, float lambda, int m)
 BOOL LCMSEXPORT cmsSmoothGamma(LPGAMMATABLE Tab, double lambda)
 
 {
-	vec w, y, z;
-	int i, nItems, Zeros, Poles;
+    vec w, y, z;
+    int i, nItems, Zeros, Poles;
 
 
-	if (cmsIsLinear(Tab->GammaTable, Tab->nEntries)) return FALSE; // Nothing to do
+    if (cmsIsLinear(Tab->GammaTable, Tab->nEntries)) return FALSE; // Nothing to do
 
     nItems = Tab -> nEntries;
 
     if (nItems > MAX_KNOTS) {
                 cmsSignalError(LCMS_ERRC_ABORTED, "cmsSmoothGamma: too many points.");
-				return FALSE;
+                return FALSE;
                 }
 
-	ZeroMemory(w, nItems * sizeof(float));
-	ZeroMemory(y, nItems * sizeof(float));
-	ZeroMemory(z, nItems * sizeof(float));
+    ZeroMemory(w, nItems * sizeof(float));
+    ZeroMemory(y, nItems * sizeof(float));
+    ZeroMemory(z, nItems * sizeof(float));
 
-	for (i=0; i < nItems; i++)
-	{
-		y[i+1] = (float) Tab -> GammaTable[i];
-		w[i+1] = 1.0;
-	}
+    for (i=0; i < nItems; i++)
+    {
+        y[i+1] = (float) Tab -> GammaTable[i];
+        w[i+1] = 1.0;
+    }
 
-	smooth2(w, y, z, (float) lambda, nItems);
+    smooth2(w, y, z, (float) lambda, nItems);
 
-	// Do some reality - checking...
-	Zeros = Poles = 0;
-	for (i=nItems; i > 1; --i) {
+    // Do some reality - checking...
+    Zeros = Poles = 0;
+    for (i=nItems; i > 1; --i) {
 
-			if (z[i] == 0.) Zeros++;
-			if (z[i] >= 65535.) Poles++;
-			if (z[i] < z[i-1]) return FALSE; // Non-Monotonic
-	}
+            if (z[i] == 0.) Zeros++;
+            if (z[i] >= 65535.) Poles++;
+            if (z[i] < z[i-1]) return FALSE; // Non-Monotonic
+    }
 
-	if (Zeros > (nItems / 3)) return FALSE;  // Degenerated, mostly zeros
-	if (Poles > (nItems / 3)) return FALSE;	 // Degenerated, mostly poles
+    if (Zeros > (nItems / 3)) return FALSE;  // Degenerated, mostly zeros
+    if (Poles > (nItems / 3)) return FALSE;  // Degenerated, mostly poles
 
-	// Seems ok
+    // Seems ok
 
-	for (i=0; i < nItems; i++) {
+    for (i=0; i < nItems; i++) {
 
-		// Clamp to WORD
+        // Clamp to WORD
 
-		float v = z[i+1];
+        float v = z[i+1];
 
-		if (v < 0) v = 0;
-		if (v > 65535.) v = 65535.;
+        if (v < 0) v = 0;
+        if (v > 65535.) v = 65535.;
 
-		Tab -> GammaTable[i] = (WORD) floor(v + .5);
+        Tab -> GammaTable[i] = (WORD) floor(v + .5);
         }
 
-	return TRUE;
+    return TRUE;
 }
 
 
 // Check if curve is exponential, return gamma if so.
 
-double LCMSEXPORT cmsEstimateGamma(LPGAMMATABLE t)
+double LCMSEXPORT cmsEstimateGammaEx(LPWORD GammaTable, int nEntries, double Thereshold)
 {
     double gamma, sum, sum2;
     double n, x, y, Std;
@@ -555,14 +597,14 @@ double LCMSEXPORT cmsEstimateGamma(LPGAMMATABLE t)
 
     sum = sum2 = n = 0;
     
-    // Does exclude endpoints	
-    for (i=1; i < t ->nEntries - 1; i++) {
+    // Does exclude endpoints   
+    for (i=1; i < nEntries - 1; i++) {
 
-            x = (double) i / (t -> nEntries - 1);
-            y = (double) t -> GammaTable[i] / 65535.;
+            x = (double) i / (nEntries - 1);
+            y = (double) GammaTable[i] / 65535.;
             
-			// Avoid 7% on lower part to prevent 
-			// artifacts due to linear ramps
+            // Avoid 7% on lower part to prevent 
+            // artifacts due to linear ramps
 
             if (y > 0. && y < 1. && x > 0.07) {
 
@@ -578,13 +620,18 @@ double LCMSEXPORT cmsEstimateGamma(LPGAMMATABLE t)
     Std = sqrt((n * sum2 - sum * sum) / (n*(n-1)));
     
 
-    if (Std > .7)
+    if (Std > Thereshold)
         return -1.0;
 
     return (sum / n);   // The mean
 }
 
 
+double LCMSEXPORT cmsEstimateGamma(LPGAMMATABLE t)
+{
+        return cmsEstimateGammaEx(t->GammaTable, t->nEntries, 0.7);
+}
+
 
 // -----------------------------------------------------------------Sampled curves
 
@@ -592,29 +639,29 @@ double LCMSEXPORT cmsEstimateGamma(LPGAMMATABLE t)
 
 LPSAMPLEDCURVE cmsAllocSampledCurve(int nItems)
 {
-	LPSAMPLEDCURVE pOut;
+    LPSAMPLEDCURVE pOut;
 
-	pOut = (LPSAMPLEDCURVE) malloc(sizeof(SAMPLEDCURVE));
-	if (pOut == NULL)
-			return NULL;
+    pOut = (LPSAMPLEDCURVE) malloc(sizeof(SAMPLEDCURVE));
+    if (pOut == NULL)
+            return NULL;
 
-	if((pOut->Values = (double *) malloc(nItems * sizeof(double))) == NULL)
-	{
-		free(pOut);
-		return NULL;
-	}
+    if((pOut->Values = (double *) malloc(nItems * sizeof(double))) == NULL)
+    {
+        free(pOut);
+        return NULL;
+    }
 
-	pOut->nItems = nItems;
-	ZeroMemory(pOut->Values, nItems * sizeof(double));
+    pOut->nItems = nItems;
+    ZeroMemory(pOut->Values, nItems * sizeof(double));
 
-	return pOut;
+    return pOut;
 }
 
 
 void cmsFreeSampledCurve(LPSAMPLEDCURVE p)
 {
-	free((LPVOID) p -> Values);
-	free((LPVOID) p);
+    free((LPVOID) p -> Values);
+    free((LPVOID) p);
 }
 
 
@@ -623,14 +670,14 @@ void cmsFreeSampledCurve(LPSAMPLEDCURVE p)
 
 LPSAMPLEDCURVE cmsDupSampledCurve(LPSAMPLEDCURVE p)
 {
-	LPSAMPLEDCURVE out;
+    LPSAMPLEDCURVE out;
 
-	out = cmsAllocSampledCurve(p -> nItems);
-	if (!out) return NULL;
+    out = cmsAllocSampledCurve(p -> nItems);
+    if (!out) return NULL;
 
-	CopyMemory(out ->Values, p ->Values, p->nItems * sizeof(double));
+    CopyMemory(out ->Values, p ->Values, p->nItems * sizeof(double));
 
-	return out;
+    return out;
 }
 
 
@@ -654,8 +701,8 @@ void cmsEndpointsOfSampledCurve(LPSAMPLEDCURVE p, double* Min, double* Max)
                         *Max = v;
         }
 
-		if (*Min < 0) *Min = 0;
-		if (*Max > 65535.0) *Max = 65535.0;
+        if (*Min < 0) *Min = 0;
+        if (*Max > 65535.0) *Max = 65535.0;
 }
 
 // Clamps to Min, Max
@@ -687,37 +734,37 @@ void cmsClampSampledCurve(LPSAMPLEDCURVE p, double Min, double Max)
 
 BOOL cmsSmoothSampledCurve(LPSAMPLEDCURVE Tab, double lambda)
 {
-	vec w, y, z;
-	int i, nItems;
+    vec w, y, z;
+    int i, nItems;
 
     nItems = Tab -> nItems;
 
     if (nItems > MAX_KNOTS) {
                 cmsSignalError(LCMS_ERRC_ABORTED, "cmsSmoothSampledCurve: too many points.");
-				return FALSE;
+                return FALSE;
                 }
 
-	ZeroMemory(w, nItems * sizeof(float));
-	ZeroMemory(y, nItems * sizeof(float));
-	ZeroMemory(z, nItems * sizeof(float));
+    ZeroMemory(w, nItems * sizeof(float));
+    ZeroMemory(y, nItems * sizeof(float));
+    ZeroMemory(z, nItems * sizeof(float));
 
-	for (i=0; i < nItems; i++)
-	{
-		float value = (float) Tab -> Values[i];
+    for (i=0; i < nItems; i++)
+    {
+        float value = (float) Tab -> Values[i];
 
-		y[i+1] = value;
-		w[i+1] = (float) ((value < 0.0) ?  0 : 1);
-	}
+        y[i+1] = value;
+        w[i+1] = (float) ((value < 0.0) ?  0 : 1);
+    }
 
 
-	smooth2(w, y, z, (float) lambda, nItems);
+    smooth2(w, y, z, (float) lambda, nItems);
 
-	for (i=0; i < nItems; i++) {
+    for (i=0; i < nItems; i++) {
 
-		Tab -> Values[i] = z[i+1];;
+        Tab -> Values[i] = z[i+1];;
      }
 
-	return TRUE;
+    return TRUE;
 
 }
 
@@ -731,8 +778,8 @@ double ScaleVal(double v, double Min, double Max, int nPoints)
 
         double a, b;
 
-		if (v < 0) return 0;
-	
+        if (v < 0) return 0;
+    
         a = (double) (nPoints - 1) / (Max - Min);
         b = a * Min;
 
@@ -758,109 +805,109 @@ void cmsRescaleSampledCurve(LPSAMPLEDCURVE p, double Min, double Max, int nPoint
 }
 
 
-// Joins two sampled curves for X and Y
+// Joins two sampled curves for X and Y. Curves should be sorted.
 
 LPSAMPLEDCURVE cmsJoinSampledCurves(LPSAMPLEDCURVE X, LPSAMPLEDCURVE Y, int nResultingPoints)
 {
-	int i, j;	
-	LPSAMPLEDCURVE out;
+    int i, j;   
+    LPSAMPLEDCURVE out;
     double MinX, MinY, MaxX, MaxY;
-	double x, y, x1, y1, x2, y2, a, b;
+    double x, y, x1, y1, x2, y2, a, b;
 
-	out = cmsAllocSampledCurve(nResultingPoints);
+    out = cmsAllocSampledCurve(nResultingPoints);
     if (out == NULL)
-		return NULL;
+        return NULL;
 
     if (X -> nItems != Y -> nItems) {
 
-		cmsSignalError(LCMS_ERRC_ABORTED, "cmsJoinSampledCurves: invalid curve.");
-		cmsFreeSampledCurve(out);
-		return NULL;
+        cmsSignalError(LCMS_ERRC_ABORTED, "cmsJoinSampledCurves: invalid curve.");
+        cmsFreeSampledCurve(out);
+        return NULL;
     }
 
-	// Get endpoints of sampled curves
+    // Get endpoints of sampled curves
     cmsEndpointsOfSampledCurve(X, &MinX, &MaxX);
     cmsEndpointsOfSampledCurve(Y, &MinY, &MaxY);
 
-	
-	// Set our points
-	out ->Values[0] = MinY;	
-	for (i=1; i < nResultingPoints; i++) {
+    
+    // Set our points
+    out ->Values[0] = MinY; 
+    for (i=1; i < nResultingPoints; i++) {
 
-		// Scale t to x domain
-		x = (i * (MaxX - MinX) / (nResultingPoints-1)) + MinX;
+        // Scale t to x domain
+        x = (i * (MaxX - MinX) / (nResultingPoints-1)) + MinX;
 
-		// Find interval in which t is within (always up, 
-		// since fn should be monotonic at all)
+        // Find interval in which t is within (always up, 
+        // since fn should be monotonic at all)
 
-		j = 1;
-		while ((j < X ->nItems - 1) && X ->Values[j] < x)
-			j++;
-			
-		// Now x is within X[j-1], X[j]
-		x1 = X ->Values[j-1]; x2 = X ->Values[j];
-		y1 = Y ->Values[j-1]; y2 = Y ->Values[j];
+        j = 1;
+        while ((j < X ->nItems - 1) && X ->Values[j] < x)
+            j++;
+            
+        // Now x is within X[j-1], X[j]
+        x1 = X ->Values[j-1]; x2 = X ->Values[j];
+        y1 = Y ->Values[j-1]; y2 = Y ->Values[j];
 
-		// Interpolate  the value
-		a = (y1 - y2) / (x1 - x2);
-		b = y1 - a * x1;
-		y = a* x + b;
-		
-		out ->Values[i] = y;
-	}
-	
+        // Interpolate  the value
+        a = (y1 - y2) / (x1 - x2);
+        b = y1 - a * x1;
+        y = a* x + b;
+        
+        out ->Values[i] = y;
+    }
+    
 
-	cmsClampSampledCurve(out, MinY, MaxY);
-	return out;
+    cmsClampSampledCurve(out, MinY, MaxY);
+    return out;
 }
 
 
 
-
+// Convert between curve types
 
 LPGAMMATABLE cmsConvertSampledCurveToGamma(LPSAMPLEDCURVE Sampled, double Max)
 {
-	LPGAMMATABLE Gamma;
-	int i, nPoints;
-	
+    LPGAMMATABLE Gamma;
+    int i, nPoints;
+    
 
-// 	cmsEndpointsOfSampledCurve(Sampled, &Min, &Max);
+    nPoints = Sampled ->nItems;
 
-	nPoints = Sampled ->nItems;
+    Gamma = cmsAllocGamma(nPoints);
+    for (i=0; i < nPoints; i++) {
+        
+        Gamma->GammaTable[i] = (WORD) floor(ScaleVal(Sampled ->Values[i], 0, Max, 65536) + .5);
+    }
 
-	Gamma = cmsAllocGamma(nPoints);
-	for (i=0; i < nPoints; i++) {
-		
-		Gamma->GammaTable[i] = (WORD) floor(ScaleVal(Sampled ->Values[i], 0, Max, 65536) + .5);
-	}
-
-	return Gamma;
+    return Gamma;
 
 }
 
+// Inverse of anterior
+
 LPSAMPLEDCURVE cmsConvertGammaToSampledCurve(LPGAMMATABLE Gamma, int nPoints)
 {
-	LPSAMPLEDCURVE Sampled;
-	L16PARAMS L16;
-	int i;
-	WORD wQuant, wValIn;
+    LPSAMPLEDCURVE Sampled;
+    L16PARAMS L16;
+    int i;
+    WORD wQuant, wValIn;
 
-	if (nPoints > 4096) {
+    if (nPoints > 4096) {
 
-		cmsSignalError(-1, "cmsConvertGammaToSampledCurve: too many points (max=4096)");
-		return NULL;
-	}
+        cmsSignalError(LCMS_ERRC_ABORTED, "cmsConvertGammaToSampledCurve: too many points (max=4096)");
+        return NULL;
+    }
 
-	cmsCalcL16Params(Gamma -> nEntries, &L16);
+    cmsCalcL16Params(Gamma -> nEntries, &L16);
        
-	Sampled = cmsAllocSampledCurve(nPoints);
-	for (i=0; i < nPoints; i++) {
-			wQuant  = _cmsQuantizeVal(i, nPoints);
-			wValIn  = cmsLinearInterpLUT16(wQuant, Gamma ->GammaTable, &L16);
-			Sampled ->Values[i] = (float) wValIn;
-	}
+    Sampled = cmsAllocSampledCurve(nPoints);
+    for (i=0; i < nPoints; i++) {
+            wQuant  = _cmsQuantizeVal(i, nPoints);
+            wValIn  = cmsLinearInterpLUT16(wQuant, Gamma ->GammaTable, &L16);
+            Sampled ->Values[i] = (float) wValIn;
+    }
 
-	return Sampled;
+    return Sampled;
 }
 
 
@@ -868,8 +915,8 @@ LPSAMPLEDCURVE cmsConvertGammaToSampledCurve(LPGAMMATABLE Gamma, int nPoints)
 static
 void ASAVE(LPWORD Table, int nEntries, const char* dump)
 {
-	FILE* f;
-	int i;
+    FILE* f;
+    int i;
 
         f = fopen(dump, "wt");
         if (!f)
@@ -877,11 +924,11 @@ void ASAVE(LPWORD Table, int nEntries, const char* dump)
 
         
 
-	for (i=0; i < nEntries; i++)
-		fprintf(f, "%g\n", (double) Table[i]);
+    for (i=0; i < nEntries; i++)
+        fprintf(f, "%g\n", (double) Table[i]);
         
 
-	fclose(f);
+    fclose(f);
 }
 #endif
 
@@ -890,65 +937,65 @@ void ASAVE(LPWORD Table, int nEntries, const char* dump)
 
 BOOL _cmsSmoothEndpoints(LPWORD Table, int nEntries)
 {
-	vec w, y, z;
-	int i, Zeros, Poles;
+    vec w, y, z;
+    int i, Zeros, Poles;
 
 #ifdef DEBUG
-		ASAVE(Table, nEntries, "nonsmt.txt");
+        ASAVE(Table, nEntries, "nonsmt.txt");
 #endif
 
 
-	if (cmsIsLinear(Table, nEntries)) return FALSE; // Nothing to do
+    if (cmsIsLinear(Table, nEntries)) return FALSE; // Nothing to do
 
     
     if (nEntries > MAX_KNOTS) {
                 cmsSignalError(LCMS_ERRC_ABORTED, "_cmsSmoothEndpoints: too many points.");
-				return FALSE;
+                return FALSE;
                 }
 
-	ZeroMemory(w, nEntries * sizeof(float));
-	ZeroMemory(y, nEntries * sizeof(float));
-	ZeroMemory(z, nEntries * sizeof(float));
+    ZeroMemory(w, nEntries * sizeof(float));
+    ZeroMemory(y, nEntries * sizeof(float));
+    ZeroMemory(z, nEntries * sizeof(float));
 
-	for (i=0; i < nEntries; i++)
-	{
-		y[i+1] = (float) Table[i];
-		w[i+1] = 1.0;
-	}
+    for (i=0; i < nEntries; i++)
+    {
+        y[i+1] = (float) Table[i];
+        w[i+1] = 1.0;
+    }
 
-	w[1]      = 65535.0;
-	w[nEntries] = 65535.0;
+    w[1]        = 65535.0;
+    w[nEntries] = 65535.0;
 
-	smooth2(w, y, z, (float) nEntries, nEntries);
+    smooth2(w, y, z, (float) nEntries, nEntries);
 
-	// Do some reality - checking...
-	Zeros = Poles = 0;
-	for (i=nEntries; i > 1; --i) {
+    // Do some reality - checking...
+    Zeros = Poles = 0;
+    for (i=nEntries; i > 1; --i) {
 
-			if (z[i] == 0.) Zeros++;
-			if (z[i] >= 65535.) Poles++;
-			if (z[i] < z[i-1]) return FALSE; // Non-Monotonic
-	}
+            if (z[i] == 0.) Zeros++;
+            if (z[i] >= 65535.) Poles++;
+            if (z[i] < z[i-1]) return FALSE; // Non-Monotonic
+    }
 
-	if (Zeros > (nEntries / 3)) return FALSE;  // Degenerated, mostly zeros
-	if (Poles > (nEntries / 3)) return FALSE;	 // Degenerated, mostly poles
+    if (Zeros > (nEntries / 3)) return FALSE;  // Degenerated, mostly zeros
+    if (Poles > (nEntries / 3)) return FALSE;    // Degenerated, mostly poles
 
-	// Seems ok
+    // Seems ok
 
-	for (i=0; i < nEntries; i++) {
+    for (i=0; i < nEntries; i++) {
 
-		// Clamp to WORD
+        // Clamp to WORD
 
-		float v = z[i+1];
+        float v = z[i+1];
 
-		if (v < 0) v = 0;
-		if (v > 65535.) v = 65535.;
+        if (v < 0) v = 0;
+        if (v > 65535.) v = 65535.;
 
-		Table[i] = (WORD) floor(v + .5);
+        Table[i] = (WORD) floor(v + .5);
         }
 
 #ifdef DEBUG
-		ASAVE(Table, nEntries, "smoothed.txt");
+        ASAVE(Table, nEntries, "smoothed.txt");
 #endif
-	return TRUE;
-}
\ No newline at end of file
+    return TRUE;
+}
