@@ -167,6 +167,7 @@ Fixed32 FixedSquare(Fixed32 a)
 // Perform (a * 65536.0) / 65535.0. This give us a fixed-point
 // value normalized into 0...1.0 range
 
+
 Fixed32 ToFixedDomain(int a)
 {
        ASM {
@@ -291,6 +292,7 @@ Fixed32 FixedSquare(Fixed32 a)
 }
 
 
+
 Fixed32 ToFixedDomain(int a)
 {
        return (Fixed32) (((double) a * 65536.0 ) / 65535.0 + 0.5);
@@ -323,6 +325,7 @@ WORD FixedScale(WORD a, Fixed32 s)
 
 #endif
 
+
 // Initiate a vector (double version)
 
 
@@ -749,3 +752,19 @@ void MAT3scaleAndCut(LPWMAT3 r, LPMAT3 v, double d)
        VEC3scaleAndCut(&r -> v[1], &v -> v[1], d);
        VEC3scaleAndCut(&r -> v[2], &v -> v[2], d);
 }
+
+
+// Saturate a vector into 0..1.0 range
+
+void VEC3saturate(LPVEC3 v)
+{
+	int i;
+	for (i=0; i < 3; i++) {
+		if (v ->n[i] < 0)
+				v ->n[i] = 0;
+		else
+		if (v ->n[i] > 1.0)
+				v ->n[i] = 1.0;
+	}
+}
+
