@@ -45,6 +45,25 @@ WORD _cmsQuantizeVal(double i, int MaxSamples)
 }
 
 
+// Is a table linear?
+
+int cmsIsLinear(WORD Table[], int nEntries)
+{
+       register int i;
+       int diff;
+
+       for (i=0; i < nEntries; i++) {
+
+           diff = abs((int) Table[i] - (int) _cmsQuantizeVal(i, nEntries));              
+           if (diff > 3)
+                     return 0;
+       }
+
+       return 1;
+}
+
+
+
 // pow() restricted to integer
 
 static
@@ -94,7 +113,7 @@ BOOL LCMSEXPORT cmsSample3DGrid(LPLUT Lut, _cmsSAMPLER Sampler, LPVOID Cargo, DW
         }
 
 
-        if (dwFlags & LUT_HASTL1) {
+        if (dwFlags & SAMPLER_HASTL1) {
 
                  for (t=0; t < (int) Lut -> InputChan; t++)
                      In[t] = cmsReverseLinearInterpLUT16(In[t],
@@ -102,21 +121,33 @@ BOOL LCMSEXPORT cmsSample3DGrid(LPLUT Lut, _cmsSAMPLER Sampler, LPVOID Cargo, DW
                                                 &Lut -> In16params);
         }
 
+
+        if (dwFlags & SAMPLER_INSPECT) {
+
+             for (t=0; t < (int) Lut -> OutputChan; t++)
+                        Out[t] = Lut->T[index + t];
+        }
+
+
         if (!Sampler(In, Out, Cargo))
                 return FALSE;
 
-                
-        if (dwFlags & LUT_HASTL2) {
+        if (!(dwFlags & SAMPLER_INSPECT)) {
+            
+            if (dwFlags & SAMPLER_HASTL2) {
+
                 for (t=0; t < (int) Lut -> OutputChan; t++)
                      Out[t] = cmsReverseLinearInterpLUT16(Out[t],
                                                    Lut -> L2[t],
                                                    &Lut -> Out16params);
                 }
 
-                
-        for (t=0; t < (int) Lut -> OutputChan; t++)
+        
+            for (t=0; t < (int) Lut -> OutputChan; t++)
                         Lut->T[index + t] = Out[t];
 
+        }
+
         index += Lut -> OutputChan;
 
     }
@@ -137,59 +168,72 @@ int XFormSampler(register WORD In[], register WORD Out[], register LPVOID Cargo)
 }
 
 
-static
-int NumOfGridPoints(int Precission, int ChannelsIn)
+
+// choose reasonable resolution
+int _cmsReasonableGridpointsByColorspace(icColorSpaceSignature Colorspace, DWORD dwFlags)
 {
 
-    int nGridPoints = 33;
+    int nChannels = _cmsChannelsOf(Colorspace);
 
-    switch (Precission) {
+    // HighResPrecalc is maximum resolution
 
-       default: 
-       case 0: 
-           
-            if (ChannelsIn > 4)
-                nGridPoints = 6;
-            else
-            if (ChannelsIn > 3)
-                nGridPoints = 17;
-            else
-                nGridPoints = 33;
-            break;
+    if (dwFlags & cmsFLAGS_HIGHRESPRECALC) {
+
+        if (nChannels > 4) 
+                return 7;       // 7 for Hifi
+
+        if (nChannels == 4)     // 23 for CMYK
+                return 23;
+    
+        return 48;      // 48 for RGB and others        
+    }
 
 
-       case 1:
-            if (ChannelsIn > 4)
-                nGridPoints = 7;
-            else
-            if (ChannelsIn > 3)
-                nGridPoints = 33;
-            else
-                nGridPoints = 48;
-            break;
-       }
+    // LowResPrecal is stripped resolution
 
-    return nGridPoints;
+    if (dwFlags & cmsFLAGS_LOWRESPRECALC) {
+        
+        if (nChannels > 4) 
+                return 6;       // 7 for Hifi
+
+        if (nChannels == 1) 
+                return 33;      // For monochrome
+
+        return 17;              // 17 for remaining
+    }
+
+    // Default values
+
+    if (nChannels > 4) 
+                return 7;       // 7 for Hifi
+
+    if (nChannels == 4)
+                return 17;      // 17 for CMYK
+
+    return 33;                  // 33 for RGB
+    
 }
 
 
-
 // This routine does compute the devicelink CLUT containing whole
 // transform. Handles any channel number.
 
-BOOL _cmsPrecalculateDeviceLink(_LPcmsTRANSFORM p, int Precission)
+LPLUT _cmsPrecalculateDeviceLink(cmsHTRANSFORM h, DWORD dwFlags)
 {
+        _LPcmsTRANSFORM p = (_LPcmsTRANSFORM) h;
        LPLUT Grid;
        int nGridPoints;
        DWORD dwFormatIn, dwFormatOut;
        int ChannelsIn, ChannelsOut;
+    
        
-       ChannelsIn   = _cmsChannelsOf(cmsGetColorSpace(p->InputProfile));
-       ChannelsOut  = _cmsChannelsOf(cmsGetColorSpace(p->OutputProfile));
+       ChannelsIn   = _cmsChannelsOf(p -> EntryColorSpace);
+       ChannelsOut  = _cmsChannelsOf(p -> ExitColorSpace);
+          
 
-       nGridPoints = NumOfGridPoints(Precission, ChannelsIn);          
+       nGridPoints = _cmsReasonableGridpointsByColorspace(p -> EntryColorSpace, dwFlags);
        Grid =  cmsAllocLUT();
-       if (!Grid) return FALSE;
+       if (!Grid) return NULL;
 
        Grid = cmsAlloc3DGrid(Grid, nGridPoints, ChannelsIn, ChannelsOut);
 
@@ -197,15 +241,16 @@ BOOL _cmsPrecalculateDeviceLink(_LPcmsTRANSFORM p, int Precission)
        dwFormatIn   = (CHANNELS_SH(ChannelsIn)|BYTES_SH(2));
        dwFormatOut  = (CHANNELS_SH(ChannelsOut)|BYTES_SH(2));
 
-       p -> FromInput = _cmsIdentifyInputFormat(dwFormatIn);
-       p -> ToOutput  = _cmsIdentifyOutputFormat(dwFormatOut);
-
+       p -> FromInput = _cmsIdentifyInputFormat(p, dwFormatIn);
+       p -> ToOutput  = _cmsIdentifyOutputFormat(p, dwFormatOut);
 
        // Fix gamut & gamma possible mismatches. 
-       // Only operates on same colorspace
+       // Only operates on RGB to RGB!!
            
-       if (cmsGetColorSpace(p->InputProfile) == cmsGetColorSpace(p->OutputProfile))
-                    _cmsComputePrelinearizationTablesFromXFORM((cmsHTRANSFORM) p, Grid);
+       if (p ->EntryColorSpace == icSigRgbData && 
+           p ->ExitColorSpace  == icSigRgbData &&
+           !(dwFlags & cmsFLAGS_NOPRELINEARIZATION))
+                _cmsComputePrelinearizationTablesFromXFORM((cmsHTRANSFORM) &p, 1, Grid);
         
             
        // Attention to this typecast! we can take the luxury to
@@ -215,145 +260,10 @@ BOOL _cmsPrecalculateDeviceLink(_LPcmsTRANSFORM p, int Precission)
        if (!cmsSample3DGrid(Grid, XFormSampler, (LPVOID) p, Grid -> wFlags)) {
 
                 cmsFreeLUT(Grid);
-                return FALSE;
+                return NULL;
        }
-
-       // All ok, store the newly created LUT
-       p -> DeviceLink = Grid;
-       return TRUE;
-}
-
-
-static
-LPLUT Create3x3EmptyLUT(void)
-{
-        LPLUT AToB0 = cmsAllocLUT();
-        AToB0 -> InputChan = AToB0 -> OutputChan = 3;
-
-        return AToB0;
-}
-
-
-// Creates a fake Lab identity.
-cmsHPROFILE LCMSEXPORT cmsCreateLabProfile(LPcmsCIExyY WhitePoint)
-{
-        cmsHPROFILE hProfile;        
-        LPLUT Lut;
-
-
-        hProfile = cmsCreateRGBProfile(WhitePoint == NULL ? cmsD50_xyY() : WhitePoint, NULL, NULL);
-
-        cmsSetDeviceClass(hProfile, icSigAbstractClass);
-        cmsSetColorSpace(hProfile, icSigLabData);
-        cmsSetPCS(hProfile,  icSigLabData);
       
-       _cmsAddTextTag(hProfile, icSigProfileDescriptionTag, "Lab (built-in)");
-       _cmsAddTextTag(hProfile, icSigDeviceMfgDescTag,      "(lcms internal)");
-       _cmsAddTextTag(hProfile, icSigDeviceModelDescTag,    "(none)");
-
-
-       // An empty LUTs is all we need
-       Lut = Create3x3EmptyLUT();
-       if (Lut == NULL) return NULL;
-
-       _cmsAddLUTTag(hProfile, icSigAToB0Tag,    Lut);
-       _cmsAddLUTTag(hProfile, icSigBToA0Tag,    Lut);
-    
-
-       cmsFreeLUT(Lut);
-
-       return hProfile;
+      
+       return Grid;
 }
 
-
-// Creates a fake XYZ identity
-cmsHPROFILE LCMSEXPORT cmsCreateXYZProfile(void)
-{
-        cmsHPROFILE hProfile;       
-        LPLUT Lut;
-
-        hProfile = cmsCreateRGBProfile(cmsD50_xyY(), NULL, NULL);
-
-        cmsSetDeviceClass(hProfile, icSigAbstractClass);
-        cmsSetColorSpace(hProfile, icSigXYZData);
-        cmsSetPCS(hProfile,  icSigXYZData);
-
-       _cmsAddTextTag(hProfile, icSigProfileDescriptionTag, "lcms XYZ identity");
-       _cmsAddTextTag(hProfile, icSigDeviceMfgDescTag,      "(lcms internal)");
-       _cmsAddTextTag(hProfile, icSigDeviceModelDescTag,    "(none)");
-
-
-       // An empty LUTs is all we need
-       Lut = Create3x3EmptyLUT();
-       if (Lut == NULL) return NULL;
-
-       _cmsAddLUTTag(hProfile, icSigAToB0Tag,    Lut);
-       _cmsAddLUTTag(hProfile, icSigBToA0Tag,    Lut);
-       _cmsAddLUTTag(hProfile, icSigPreview0Tag, Lut);
-
-       cmsFreeLUT(Lut);
-
-       _cmsAddTextTag(hProfile, icSigProfileDescriptionTag, "XYZ (built-in)");
-       return hProfile;
-}
-
-
-
-/*
-
-If  RsRGB,GsRGB, BsRGB < 0.04045
-
-    R =  RsRGB / 12.92
-    G =  GsRGB / 12.92
-    B =  BsRGB / 12.92
-
- 
-
-else if  RsRGB,GsRGB, BsRGB >= 0.04045
-
-    R = ((RsRGB + 0.055) / 1.055)^2.4
-    G = ((GsRGB + 0.055) / 1.055)^2.4
-    B = ((BsRGB + 0.055) / 1.055)^2.4
-
-  */
-
-static
-LPGAMMATABLE Build_sRGBGamma(void)
-{
-    double Parameters[5];
-
-    Parameters[0] = 2.4;
-    Parameters[1] = 1. / 1.055;
-    Parameters[2] = 0.055 / 1.055;
-    Parameters[3] = 1. / 12.92;
-    Parameters[4] = 0.04045;    // d
-
-    return cmsBuildParametricGamma(1024, 3, Parameters);
-}
-
-cmsHPROFILE LCMSEXPORT cmsCreate_sRGBProfile(void)
-{
-       cmsCIExyY       D65;
-       cmsCIExyYTRIPLE Rec709Primaries = {
-                                   {0.6400, 0.3300, 1.0},
-                                   {0.3000, 0.6000, 1.0},
-                                   {0.1500, 0.0600, 1.0}
-                                   };
-       LPGAMMATABLE Gamma22[3];
-       cmsHPROFILE  hsRGB;
- 
-       cmsWhitePointFromTemp(6504, &D65);
-       Gamma22[0] = Gamma22[1] = Gamma22[2] = Build_sRGBGamma();
-           
-       hsRGB = cmsCreateRGBProfile(&D65, &Rec709Primaries, Gamma22);
-       cmsFreeGamma(Gamma22[0]);
-
-       _cmsAddTextTag(hsRGB, icSigProfileDescriptionTag, "sRGB (built-in)");
-       _cmsAddTextTag(hsRGB, icSigDeviceMfgDescTag,       "(lcms internal)");
-       _cmsAddTextTag(hsRGB, icSigDeviceModelDescTag,    "(none)");
-
-        
-       return hsRGB;
-}
-
-
