@@ -1,6 +1,6 @@
 //
 //  Little cms
-//  Copyright (C) 1998-2001 Marti Maria
+//  Copyright (C) 1998-2002 Marti Maria
 //
 // THIS SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND,
 // EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY
@@ -28,7 +28,7 @@
 // License along with this library; if not, write to the Free Software
 // Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
-// Version 1.07
+// Version 1.11
 
 // This program does apply profiles to (some) TIFF files
 
@@ -38,7 +38,11 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <sys/stat.h>
 
+#ifndef NON_WINDOWS
+#include <io.h>
+#endif
 
 #include "tiffio.h"
 
@@ -47,22 +51,34 @@
 
 extern int   xoptind;
 extern char *xoptarg;
-extern int   xoperr;
+extern int   xopterr;
+extern char   SW;
 int    cdecl xgetopt(int argc, char *argv[], char *optionS);
 
 // ------------------------------------------------------------------------
 
+#ifdef NON_WINDOWS
+#  ifndef stricmp
+#     define stricmp strcasecmp
+#  endif
+#endif
+
 // Flags
 
 static BOOL Verbose                = FALSE;
 static BOOL BlackWhiteCompensation = FALSE;
-static BOOL IgnoreEmbedded        = FALSE;
-static BOOL Width16               = FALSE;
-static BOOL GamutCheck            = FALSE;
-static int Intent                 = INTENT_PERCEPTUAL;
-static int PrecalcMode            = 1;
-static char *cInpProf  = "sRGB Color Space Profile.icm";
-static char *cOutProf  = "sRGB Color Space Profile.icm";
+static BOOL IgnoreEmbedded         = FALSE;
+static BOOL EmbedProfile           = FALSE;
+static BOOL Width16                = FALSE;
+static BOOL GamutCheck             = FALSE;
+static BOOL lIsDeviceLink          = FALSE;
+static int Intent                  = INTENT_PERCEPTUAL;
+static int ProofingIntent          = INTENT_PERCEPTUAL;
+static int PrecalcMode             = 1;
+static double InkLimit             = 400;
+
+static char *cInpProf  = NULL;
+static char *cOutProf  = NULL;
 static char *cProofing = NULL;
 
 
@@ -74,14 +90,15 @@ void ConsoleWarningHandler(const char* module, const char* fmt, va_list ap)
 {
         char e[512] = { '\0' };
         if (module != NULL)
-              strcat(strcpy(e, module), ":");
+              strcat(strcpy(e, module), ": ");
 
         vsprintf(e+strlen(e), fmt, ap);
         strcat(e, ".");
         if (Verbose) {
 
-              fprintf(stderr, "\nWarning ");
-              fprintf(stderr, "- %s\n", e);
+              fprintf(stderr, "\nWarning");
+              fprintf(stderr, " %s\n", e);
+              fflush(stderr);
               }
 }
 
@@ -91,12 +108,13 @@ void ConsoleErrorHandler(const char* module, const char* fmt, va_list ap)
        char e[512] = { '\0' };
 
        if (module != NULL)
-              strcat(strcpy(e, module), ":");
+              strcat(strcpy(e, module), ": ");
 
        vsprintf(e+strlen(e), fmt, ap);
        strcat(e, ".");
-       fprintf(stderr, "\nError ");
-       fprintf(stderr, "- %s\n", e);
+       fprintf(stderr, "\nError");
+       fprintf(stderr, " %s\n", e);
+       fflush(stderr);
 }
 
 
@@ -134,7 +152,7 @@ DWORD GetInputPixelType(TIFF *Bank)
      int ColorChannels, IsPlanar = 0, pt = 0;
 
      TIFFGetField(Bank,           TIFFTAG_PHOTOMETRIC,   &Photometric);
-     TIFFGetFieldDefaulted(Bank, TIFFTAG_BITSPERSAMPLE, &bps);
+     TIFFGetFieldDefaulted(Bank,  TIFFTAG_BITSPERSAMPLE, &bps);
 
      if (bps == 1)
        FatalError("Sorry, bilevel TIFFs has nothig to do with ICC profiles");
@@ -151,7 +169,7 @@ DWORD GetInputPixelType(TIFF *Bank)
      case PLANARCONFIG_SEPARATE: IsPlanar = 1; break;
      default:
 
-       FatalError("Unsupported planar configuration (=%d) ", (int) PlanarConfig);
+     FatalError("Unsupported planar configuration (=%d) ", (int) PlanarConfig);
      }
 
      // If Samples per pixel == 1, PlanarConfiguration is irrelevant and need
@@ -168,62 +186,75 @@ DWORD GetInputPixelType(TIFF *Bank)
      switch (Photometric) {
 
      case PHOTOMETRIC_MINISWHITE:
-                                   reverse = 1;
+                                   
+            reverse = 1;
+
      case PHOTOMETRIC_MINISBLACK:
-                                   pt = PT_GRAY;
-                                   break;
+                                   
+            pt = PT_GRAY;                                
+            break;
 
      case PHOTOMETRIC_RGB:
-                                   pt = PT_RGB;
-                                   break;
+                                   
+            pt = PT_RGB;
+            break;
 
 
      case PHOTOMETRIC_PALETTE:
-                                             FatalError("Sorry, palette images not supported (at least on this version)"); 
+                                             
+            FatalError("Sorry, palette images not supported (at least on this version)"); 
+
      case PHOTOMETRIC_SEPARATED:
-                                   if (ColorChannels == 4)
-                                          pt = PT_CMYK;
-                                   else
-                                   if (ColorChannels == 3)
-                                          pt = PT_CMY;
-                                   else
-                                          FatalError("Sorry, HiFi separations can't be restored");
-                                   break;
+           if (ColorChannels == 4)
+                  pt = PT_CMYK;
+           else
+           if (ColorChannels == 3)
+                  pt = PT_CMY;
+           else
+           if (ColorChannels == 6)
+                  pt = PT_HiFi;
+           else
+           if (ColorChannels == 8)
+                  pt = PT_HiFi8;           
+           else
+                  FatalError("What a weird separation of %d channels?!?!", ColorChannels);
+           break;
 
      case PHOTOMETRIC_YCBCR:
-                                   TIFFGetField(Bank, TIFFTAG_COMPRESSION, &Compression);
-                                   {
-                                          uint16 subx, suby;
+           TIFFGetField(Bank, TIFFTAG_COMPRESSION, &Compression);
+           {
+                  uint16 subx, suby;
 
-                                          pt = PT_YCbCr;
-                                          TIFFGetFieldDefaulted(Bank, TIFFTAG_YCBCRSUBSAMPLING, &subx, &suby);
-                                          if (subx != 1 || suby != 1)
-                                                 FatalError("Sorry, subsampled images not supported");
+                  pt = PT_YCbCr;
+                  TIFFGetFieldDefaulted(Bank, TIFFTAG_YCBCRSUBSAMPLING, &subx, &suby);
+                  if (subx != 1 || suby != 1)
+                         FatalError("Sorry, subsampled images not supported");
 
-                                   }
-                                   break;
+           }
+           break;
 
+     case 9:
      case PHOTOMETRIC_CIELAB:
-                                   pt = PT_Lab;
-                                   break;
+           pt = PT_Lab;
+           break;
 
      case PHOTOMETRIC_LOGL:     /* CIE Log2(L) */
-                                   FatalError("Hummm... I have been unable of find any of  these, please contact me at marti@littlecms.com, thanx.");
+           FatalError("Hummm... I have been unable of find any of  these, please contact me at marti@littlecms.com, thanx.");
 
      case PHOTOMETRIC_LOGLUV:      /* CIE Log2(L) (u',v') */
 
-                                   TIFFSetField(Bank, TIFFTAG_SGILOGDATAFMT, SGILOGDATAFMT_16BIT);
-                                   pt = PT_YUV;             // *ICCSpace = icSigLuvData;
-                                   bps = 16;             // 16 bits forced by LibTiff
-                                   break;
+           TIFFSetField(Bank, TIFFTAG_SGILOGDATAFMT, SGILOGDATAFMT_16BIT);
+           pt = PT_YUV;             // *ICCSpace = icSigLuvData;
+           bps = 16;               // 16 bits forced by LibTiff
+           break;
 
      default:
-              FatalError("Unsupported TIFF color space (Photometric %d)", Photometric);
+           FatalError("Unsupported TIFF color space (Photometric %d)", Photometric);
      }
 
      // Convert bits per sample to bytes per sample
 
-     bps >>= 3; /* /= 8; */
+     bps >>= 3; 
 
      return (COLORSPACE_SH(pt)|PLANAR_SH(IsPlanar)|EXTRA_SH(extra)|CHANNELS_SH(ColorChannels)|BYTES_SH(bps)|FLAVOR_SH(reverse));
 }
@@ -248,13 +279,20 @@ DWORD ComputeOutputFormatDescriptor(DWORD dwInput, int OutColorSpace, int bps)
    case PT_Lab:
    case PT_YUV:
    case PT_YCbCr:
-           Channels = 3;
+               Channels = 3;
                break;
 
    case PT_CMYK:
-           Channels = 4;
+               Channels = 4;
                break;
 
+   case  PT_HiFi:
+               Channels = 6;
+               break;
+
+   case  PT_HiFi8:
+               Channels = 8;
+               break;
    default:
                FatalError("Unsupported output color space");
    }
@@ -265,11 +303,12 @@ DWORD ComputeOutputFormatDescriptor(DWORD dwInput, int OutColorSpace, int bps)
 
 // Equivalence between ICC color spaces and lcms color spaces
 
-static
-int GetProfileColorSpace(cmsHPROFILE hProfile)
-{
-    icColorSpaceSignature ProfileSpace = cmsGetColorSpace(hProfile);
 
+
+static
+int ICC2LCMS(icColorSpaceSignature ProfileSpace)
+{
+    
        switch (ProfileSpace) {
 
        case icSigGrayData: return  PT_GRAY;
@@ -285,50 +324,25 @@ int GetProfileColorSpace(cmsHPROFILE hProfile)
        case icSigHlsData:  return  PT_HLS;
        case icSigYxyData:  return  PT_Yxy;
 
-       default:  return icMaxEnumData;
-       }
-}
-
-
-static
-icColorSpaceSignature GetICCcolorSpace(int OurNotation)
-{
-       switch (OurNotation) {
-
-       case 1:
-       case PT_GRAY: return  icSigGrayData;
-
-       case 2:
-       case PT_RGB:  return  icSigRgbData;
-
-       case PT_CMY:  return  icSigCmyData;
-       case PT_CMYK: return  icSigCmykData;
-       case PT_YCbCr:return  icSigYCbCrData;
-       case PT_YUV:  return  icSigLuvData;
-       case PT_XYZ:  return  icSigXYZData;
-       case PT_Lab:  return  icSigLabData;
-       case PT_YUVK: return  icSigLuvKData;
-       case PT_HSV:  return  icSigHsvData;
-       case PT_HLS:  return  icSigHlsData;
-       case PT_Yxy:  return  icSigYxyData;
+       case icSigHexachromeData: return PT_HiFi;
+       case icSig8colorData:     return PT_HiFi8;
 
        default:  return icMaxEnumData;
        }
 }
 
 
-
 // Tile based transforms
 
 static
-int TileBasedXform(cmsHTRANSFORM hXForm, TIFF* in, TIFF* out)
+int TileBasedXform(cmsHTRANSFORM hXForm, TIFF* in, TIFF* out, int nPlanes)
 {
     tsize_t BufSizeIn  = TIFFTileSize(in);
     tsize_t BufSizeOut = TIFFTileSize(out);
     unsigned char *BufferIn, *BufferOut;
-    ttile_t i, TileCount = TIFFNumberOfTiles(in);
-    uint16 tw, tl;
-    int PixelCount;
+    ttile_t i, TileCount = TIFFNumberOfTiles(in) / nPlanes;
+    uint32 tw, tl;
+    int PixelCount, j;
 
 
     TIFFGetFieldDefaulted(in, TIFFTAG_TILEWIDTH,  &tw);
@@ -336,20 +350,28 @@ int TileBasedXform(cmsHTRANSFORM hXForm, TIFF* in, TIFF* out)
 
      PixelCount = (int) tw * tl;
 
-     BufferIn = (unsigned char *) _TIFFmalloc(BufSizeIn);
-    if (!BufferIn) OutOfMem(BufSizeIn);
+     BufferIn = (unsigned char *) _TIFFmalloc(BufSizeIn * nPlanes);
+    if (!BufferIn) OutOfMem(BufSizeIn * nPlanes);
 
-    BufferOut = (unsigned char *) _TIFFmalloc(BufSizeOut);
-    if (!BufferOut) OutOfMem(BufSizeOut);
+    BufferOut = (unsigned char *) _TIFFmalloc(BufSizeOut * nPlanes);
+    if (!BufferOut) OutOfMem(BufSizeOut * nPlanes);
 
 
     for (i = 0; i < TileCount; i++) {
 
-        if (TIFFReadEncodedTile(in, i, BufferIn, BufSizeIn) < 0)   goto cleanup;
+        for (j=0; j < nPlanes; j++) {
+
+            if (TIFFReadEncodedTile(in, i + (j* TileCount), 
+                                    BufferIn + (j*BufSizeIn), BufSizeIn) < 0)   goto cleanup;
+        }
 
       cmsDoTransform(hXForm, BufferIn, BufferOut, PixelCount);
 
-        if (TIFFWriteEncodedTile(out, i, BufferOut, BufSizeOut) < 0) goto cleanup;
+      for (j=0; j < nPlanes; j++) {
+
+             if (TIFFWriteEncodedTile(out, i + (j*TileCount),
+                                    BufferOut + (j*BufSizeOut), BufSizeOut) < 0) goto cleanup;
+      }
 
     }
 
@@ -369,35 +391,48 @@ cleanup:
 // Strip based transforms
 
 static
-int StripBasedXform(cmsHTRANSFORM hXForm, TIFF* in, TIFF* out)
+int StripBasedXform(cmsHTRANSFORM hXForm, TIFF* in, TIFF* out, int nPlanes)
 {
     tsize_t BufSizeIn  = TIFFStripSize(in);
     tsize_t BufSizeOut = TIFFStripSize(out);
     unsigned char *BufferIn, *BufferOut;
-    ttile_t i, StripCount = TIFFNumberOfStrips(in);
+    ttile_t i, StripCount = TIFFNumberOfStrips(in) / nPlanes;
     uint32 sw;
     uint32 sl;
+    uint32 iml;
+    int j;
     int PixelCount;
 
     TIFFGetFieldDefaulted(in, TIFFTAG_IMAGEWIDTH,  &sw);
     TIFFGetFieldDefaulted(in, TIFFTAG_ROWSPERSTRIP, &sl);
+    TIFFGetFieldDefaulted(in, TIFFTAG_IMAGELENGTH, &iml);
 
-    PixelCount = (int) sw * sl;
+   
 
-    BufferIn = (unsigned char *) _TIFFmalloc(BufSizeIn);
-    if (!BufferIn) OutOfMem(BufSizeIn);
+    BufferIn = (unsigned char *) _TIFFmalloc(BufSizeIn * nPlanes);
+    if (!BufferIn) OutOfMem(BufSizeIn * nPlanes);
 
-    BufferOut = (unsigned char *) _TIFFmalloc(BufSizeOut);
-    if (!BufferOut) OutOfMem(BufSizeOut);
+    BufferOut = (unsigned char *) _TIFFmalloc(BufSizeOut * nPlanes);
+    if (!BufferOut) OutOfMem(BufSizeOut * nPlanes);
 
 
     for (i = 0; i < StripCount; i++) {
 
-        if (TIFFReadEncodedStrip(in, i, BufferIn, BufSizeIn) < 0)   goto cleanup;
+        for (j=0; j < nPlanes; j++) {
 
-      cmsDoTransform(hXForm, BufferIn, BufferOut, PixelCount);
+            if (TIFFReadEncodedStrip(in, i + (j * StripCount), 
+                                        BufferIn + (j * BufSizeIn), BufSizeIn) < 0)   goto cleanup;
+        }
 
-        if (TIFFWriteEncodedStrip(out, i, BufferOut, BufSizeOut) < 0) goto cleanup;
+        PixelCount = (int) sw * (iml < sl ? iml : sl);
+        iml -= sl;
+
+        cmsDoTransform(hXForm, BufferIn, BufferOut, PixelCount);
+
+        for (j=0; j < nPlanes; j++) {
+            if (TIFFWriteEncodedStrip(out, i + (j * StripCount), 
+                                     BufferOut + j * BufSizeOut, BufSizeOut) < 0) goto cleanup;
+        }
 
     }
 
@@ -441,7 +476,7 @@ void WriteOutputTags(TIFF *out, int Colorspace, int BytesPerSample)
    case PT_CMY:
            TIFFSetField(out, TIFFTAG_PHOTOMETRIC, PHOTOMETRIC_SEPARATED);
            TIFFSetField(out, TIFFTAG_SAMPLESPERPIXEL, 3);
-           TIFFSetField(out, TIFFTAG_INKSET, 0);
+           TIFFSetField(out, TIFFTAG_INKSET, 2);
            TIFFSetField(out, TIFFTAG_BITSPERSAMPLE, BitsPerSample);
            break;
 
@@ -453,12 +488,31 @@ void WriteOutputTags(TIFF *out, int Colorspace, int BytesPerSample)
            break;
 
     case PT_Lab:
-
+            if (BitsPerSample == 16) 
+            TIFFSetField(out, TIFFTAG_PHOTOMETRIC, 9);
+            else
             TIFFSetField(out, TIFFTAG_PHOTOMETRIC, PHOTOMETRIC_CIELAB);
             TIFFSetField(out, TIFFTAG_SAMPLESPERPIXEL, 3);
             TIFFSetField(out, TIFFTAG_BITSPERSAMPLE, BitsPerSample);    // Needed by TIFF Spec
             break;
-   default:
+
+     case PT_HiFi:
+
+            TIFFSetField(out, TIFFTAG_PHOTOMETRIC, PHOTOMETRIC_SEPARATED);
+            TIFFSetField(out, TIFFTAG_SAMPLESPERPIXEL, 6);
+            TIFFSetField(out, TIFFTAG_INKSET, 2);
+            TIFFSetField(out, TIFFTAG_BITSPERSAMPLE, BitsPerSample);
+            break;
+
+     case PT_HiFi8:
+
+            TIFFSetField(out, TIFFTAG_PHOTOMETRIC, PHOTOMETRIC_SEPARATED);
+            TIFFSetField(out, TIFFTAG_SAMPLESPERPIXEL, 8);
+            TIFFSetField(out, TIFFTAG_INKSET, 2);
+            TIFFSetField(out, TIFFTAG_BITSPERSAMPLE, BitsPerSample);
+            break;
+
+     default:
            FatalError("Unsupported output colorspace");
 
    }
@@ -508,6 +562,9 @@ void CopyOtherTags(TIFF* in, TIFF* out)
         CopyField(TIFFTAG_IMAGEDEPTH, longv);
         CopyField(TIFFTAG_TILEDEPTH, longv);
 
+        CopyField(TIFFTAG_TILEWIDTH,  longv);
+        CopyField(TIFFTAG_TILELENGTH, longv);
+
         CopyField(TIFFTAG_ARTIST, stringv);
         CopyField(TIFFTAG_IMAGEDESCRIPTION, stringv);
         CopyField(TIFFTAG_MAKE, stringv);
@@ -520,12 +577,77 @@ void CopyOtherTags(TIFF* in, TIFF* out)
 
 }
 
+// A replacement for (the nonstandard) filelenght
+
+static
+int xfilelength(int fd)
+{
+#ifdef _MSC_VER
+        return _filelength(fd);
+#else
+        struct stat sb;
+        if (fstat(fd, &sb) < 0)
+                return(-1);
+        return(sb.st_size);
+#endif
+
+
+}
+
+static
+void DoEmbedProfile(TIFF* Out, const char* ProfileFile)
+{
+    FILE* f;
+    size_t size, EmbedLen;
+    LPBYTE EmbedBuffer;
+
+        f = fopen(ProfileFile, "rb");
+        if (f == NULL) return;
+
+        size = xfilelength(fileno(f));
+        EmbedBuffer = (LPBYTE) malloc(size + 1);
+        EmbedLen = fread(EmbedBuffer, 1, size, f);
+        fclose(f);
+        EmbedBuffer[EmbedLen] = 0;
+
+        TIFFSetField(Out, TIFFTAG_ICCPROFILE, EmbedLen, EmbedBuffer);
+        free(EmbedBuffer);
+}
+
+
+static
+cmsHPROFILE OpenProfile(const char* File)
+{
+       if (!File) 
+            return cmsCreate_sRGBProfile();    
+       
+       if (stricmp(File, "*Lab") == 0)
+                return cmsCreateLabProfile(NULL);
+       
+       if (stricmp(File, "*XYZ") == 0)
+                return cmsCreateXYZProfile();
+         
+       if (stricmp(File, "*Gray22") == 0) {
+           LPGAMMATABLE Gamma = cmsBuildGamma(256, 2.2);
+           cmsHPROFILE hProfile = cmsCreateGrayProfile(cmsD50_xyY(), Gamma);
+           cmsFreeGamma(Gamma);
+           return hProfile;
+
+       }
+
+        return cmsOpenProfileFromFile(File, "r");
+}
+
+
+
+
+
 // Transform one image
 
 static
 int TransformImage(TIFF* in, TIFF* out, char *cDefInpProf, char *cOutProf)
 {
-       cmsHPROFILE hIn, hOut, hProof;
+       cmsHPROFILE hIn, hOut, hProof, hInkLimit = NULL;
        cmsHTRANSFORM xform;
        DWORD wInput, wOutput;
        int OutputColorSpace;
@@ -533,21 +655,26 @@ int TransformImage(TIFF* in, TIFF* out, char *cDefInpProf, char *cOutProf)
        DWORD dwFlags = 0; 
        DWORD EmbedLen;
        LPBYTE EmbedBuffer;
+       int nPlanes;
+
+       if (EmbedProfile && cOutProf) 
+           DoEmbedProfile(out, cOutProf);
 
 
-
-       if (BlackWhiteCompensation) {
-
-            dwFlags |= cmsFLAGS_WHITEBLACKCOMPENSATION;
-            if (PrecalcMode == 0) 
-                    FatalError("Cannot use White/Black compensation without precalculation. Use /c1 or /c2.");
-       }
+       
+       if (BlackWhiteCompensation) 
+            dwFlags |= cmsFLAGS_WHITEBLACKCOMPENSATION;           
+       
+       
 
        switch (PrecalcMode) {
            
        case 0: dwFlags |= cmsFLAGS_NOTPRECALC; break;
        case 2: dwFlags |= cmsFLAGS_HIGHRESPRECALC; break;
-       default:;
+       case 3: dwFlags |= cmsFLAGS_LOWRESPRECALC; break;
+       case 1: break;
+
+       default: FatalError("Unknown precalculation mode '%d'", PrecalcMode);
        }
         
 
@@ -555,24 +682,40 @@ int TransformImage(TIFF* in, TIFF* out, char *cDefInpProf, char *cOutProf)
             dwFlags |= cmsFLAGS_GAMUTCHECK;
         
 
-       if (!IgnoreEmbedded && TIFFGetField(in, TIFFTAG_ICCPROFILE, &EmbedLen, &EmbedBuffer))
-       {
-              hIn = cmsOpenProfileFromMem(EmbedBuffer, EmbedLen);
-              _TIFFfree(EmbedBuffer);
-              if (Verbose) fprintf(stdout, " (embedded profile found)");
-       }
-       else
-       {
-              hIn = cmsOpenProfileFromFile(cDefInpProf, "r");
-       }
-
-       hOut = cmsOpenProfileFromFile(cOutProf, "r");
-
-
        hProof = NULL;
-       if (cProofing != NULL) {
+       hOut = NULL;
 
-           hProof = cmsOpenProfileFromFile(cProofing, "r");
+       if (lIsDeviceLink) {
+
+            hIn = cmsOpenProfileFromFile(cDefInpProf, "r");                  
+       }
+       else {
+
+                if (!IgnoreEmbedded && 
+                    TIFFGetField(in, TIFFTAG_ICCPROFILE, &EmbedLen, &EmbedBuffer)) {
+
+                      hIn = cmsOpenProfileFromMem(EmbedBuffer, EmbedLen);
+                      if (Verbose) {
+                          fprintf(stdout, " (Embedded profile found)\n");
+                          fprintf(stdout, "Product name: %s\n", cmsTakeProductName(hIn));
+                          fprintf(stdout, "Description : %s\n", cmsTakeProductDesc(hIn));                          
+                          fflush(stdout);
+                      }
+
+                }
+                else
+                {                    
+                       hIn = OpenProfile(cDefInpProf);
+                }
+
+               
+                 hOut = OpenProfile(cOutProf);
+                 
+                 if (cProofing != NULL) {
+
+                   hProof = cmsOpenProfileFromFile(cProofing, "r");
+                   dwFlags |= cmsFLAGS_SOFTPROOFING;
+               }
        }
 
        // Take input color space
@@ -581,38 +724,80 @@ int TransformImage(TIFF* in, TIFF* out, char *cDefInpProf, char *cOutProf)
 
        // Assure both, input profile and input TIFF are on same colorspace
 
-       if (cmsGetColorSpace(hIn) != GetICCcolorSpace(T_COLORSPACE(wInput)))
+       if (cmsGetColorSpace(hIn) != _cmsICCcolorSpace(T_COLORSPACE(wInput)))
               FatalError("Input profile is not operating in proper color space");
 
-       // Output colorspace is given by output profile
-
-       OutputColorSpace = GetProfileColorSpace(hOut);
+      
+       if (!lIsDeviceLink) 
+                OutputColorSpace = ICC2LCMS(cmsGetColorSpace(hOut));
+       else 
+                OutputColorSpace = ICC2LCMS(cmsGetPCS(hIn));
+                
        wOutput      = ComputeOutputFormatDescriptor(wInput, OutputColorSpace, bps);
 
        WriteOutputTags(out, OutputColorSpace, bps);
        CopyOtherTags(in, out);
 
+       // Ink limit
+       if (InkLimit != 400.0 && 
+                (OutputColorSpace == PT_CMYK || OutputColorSpace == PT_CMY)) {
+
+           cmsHPROFILE hProfiles[10];
+           int nProfiles = 0;
+
+
+           hInkLimit = cmsCreateInkLimitingDeviceLink(cmsGetColorSpace(hOut), InkLimit);
+
+           hProfiles[nProfiles++] = hIn;
+           if (hProof) {
+                hProfiles[nProfiles++] = hProof;
+                hProfiles[nProfiles++] = hProof;
+           }
+
+           hProfiles[nProfiles++] = hOut;
+           hProfiles[nProfiles++] = hInkLimit;
+                   
+           xform = cmsCreateMultiprofileTransform(hProfiles, nProfiles, 
+                                                wInput, wOutput, Intent, dwFlags);
+           
+       }
+       else {
+
        xform = cmsCreateProofingTransform(hIn, wInput, 
                                           hOut, wOutput, 
                                           hProof, Intent, 
-                                          INTENT_ABSOLUTE_COLORIMETRIC, dwFlags);
+                                          ProofingIntent, 
+                                          dwFlags);
+       }
+
+
+       // Planar stuff
+
+       if (T_PLANAR(wInput)) 
+            nPlanes = T_CHANNELS(wInput) + T_EXTRA(wInput);
+       else
+            nPlanes = 1;
 
        // Handle tile by tile or strip by strip
 
        if (TIFFIsTiled(in)) {
 
-                TileBasedXform(xform, in, out);
+                TileBasedXform(xform, in, out, nPlanes);
        }
        else {
 
-            StripBasedXform(xform, in, out);
+                StripBasedXform(xform, in, out, nPlanes);
        }
 
 
        cmsDeleteTransform(xform);
        cmsCloseProfile(hIn);
        cmsCloseProfile(hOut);
-       if (hProof) cmsCloseProfile(hProof);
+       if (hInkLimit) 
+           cmsCloseProfile(hInkLimit);
+       if (hProof) 
+           cmsCloseProfile(hProof);
+
 
        TIFFWriteDirectory(out);
 
@@ -625,8 +810,9 @@ int TransformImage(TIFF* in, TIFF* out, char *cDefInpProf, char *cOutProf)
 static
 void Help(int level)
 {
-     fprintf(stderr, "little cms ICC profile applier for tiff  - v2.0\n\n");
-
+    fprintf(stderr, "little cms ICC profile applier for TIFF - v3.3\n\n");
+    fflush(stderr);
+    
      switch(level) {
 
      default:
@@ -635,23 +821,34 @@ void Help(int level)
      fprintf(stderr, "usage: tifficc [flags] input.tif output.tif\n");
 
      fprintf(stderr, "\nflags:\n\n");
-     fprintf(stderr, "/v - Verbose\n");
-     fprintf(stderr, "/i<profile> - Input profile (defaults to sRGB)\n");
-     fprintf(stderr, "/o<profile> - Output profile (defaults to sRGB)\n");   
-     fprintf(stderr, "/t<0,1,2,3> - Intent (0=Perceptual, 1=Colorimetric, 2=Saturation, 3=Absolute)\n");    
+     fprintf(stderr, "%cv - Verbose\n", SW);
+     fprintf(stderr, "%ci<profile> - Input profile (defaults to sRGB)\n", SW);
+     fprintf(stderr, "%co<profile> - Output profile (defaults to sRGB)\n", SW);   
+     fprintf(stderr, "%cl<profile> - Transform by device-link profile\n", SW);   
+     fprintf(stderr, "%ct<0,1,2,3> - Intent (0=Perceptual, 1=Colorimetric, 2=Saturation, 3=Absolute)\n", SW);    
      fprintf(stderr, "\n");
 
-     fprintf(stderr, "/w - Wide output (generates 16 bps tiff)\n");
-     fprintf(stderr, "/b - Black/White compensation\n");
-     fprintf(stderr, "/n - Ignore embedded profile\n");
-     fprintf(stderr, "/c<0,1,2> - Precalculates transform (0=off, 1=normal, 2=Hi-res) [defaults to 1]\n");     
+     fprintf(stderr, "%cw - Wide output (generates 16 bps tiff)\n", SW);
+    
+     fprintf(stderr, "%cn - Ignore embedded profile on input\n", SW);
+     fprintf(stderr, "%ce - Embed destination profile\n", SW);
+     fprintf(stderr, "%cc<0,1,2,3> - Precalculates transform (0=Off, 1=Normal, 2=Hi-res, 3=LoRes) [defaults to 1]\n", SW);     
      fprintf(stderr, "\n");
 
-     fprintf(stderr, "/p<profile> - Soft proof profile\n");
-     fprintf(stderr, "/g - Marks out-of-gamut colors on softproof\n");
+     fprintf(stderr, "%cp<profile> - Soft proof profile\n", SW);
+     fprintf(stderr, "%cm<0,1,2,3> - Soft proof intent\n", SW);
+     fprintf(stderr, "%cg - Marks out-of-gamut colors on softproof\n", SW);
      
+     fprintf(stderr, "\n"); 
+     fprintf(stderr, "%cb - Black point compensation\n", SW);
+     fprintf(stderr, "%ck<0..400> - Ink-limiting in %% (CMYK only)\n", SW);
+
      fprintf(stderr, "\n");
-     fprintf(stderr, "/h<0,1,2> - More help\n");
+     fprintf(stderr, "%ch<0,1,2> - More help\n", SW);
+     fprintf(stderr, "\n");
+     fprintf(stderr, "You can also use '*Lab' and '*XYZ' as predefined, built-in\n");
+     fprintf(stderr, "profiles for CIE L*a*b* and XYZ color spaces.\n");
+     
      break;
 
      case 1:
@@ -659,27 +856,31 @@ void Help(int level)
      
      fprintf(stderr, "Examples:\n\n"
                      "To color correct from scanner to sRGB:\n"
-                     "\ttifficc /iscanner.icm in.tif out.tif\n"
+                     "\ttifficc %ciscanner.icm in.tif out.tif\n"
                      "To convert from monitor1 to monitor2:\n"
-                     "\ttifficc /imon1.icm /omon2.icm in.tif out.tif\n"
+                     "\ttifficc %cimon1.icm %comon2.icm in.tif out.tif\n"
                      "To make a CMYK separation:\n"
-                     "\ttifficc /oprinter.icm inrgb.tif outcmyk.tif\n"
+                     "\ttifficc %coprinter.icm inrgb.tif outcmyk.tif\n"
                      "To recover sRGB from a CMYK separation:\n"
-                     "\ttifficc /iprinter.icm incmyk.tif outrgb.tif\n"
+                     "\ttifficc %ciprinter.icm incmyk.tif outrgb.tif\n"
                      "To convert from CIELab TIFF to sRGB\n"
-                     "\ttifficc /iTiffLab8Spac.icm in.tif out.tif\n\n");
+                     "\ttifficc %ciTiffLab8Spac.icm in.tif out.tif\n\n", 
+                     SW, SW, SW, SW, SW, SW);
      break;
 
      case 2:
 
+    
      fprintf(stderr, "This program is intended to be a demo of the little cms\n"
                      "engine. Both lcms and this program are freeware. You can\n"
                      "obtain both in source code at http://www.littlecms.com\n"
                      "For suggestions, comments, bug reports etc. send mail to\n"
-                     "marti@littlecms.com\n\n");
+                     "info@littlecms.com\n\n");
+    
      break;
      }
 
+     fflush(stderr);
      exit(0);
 }
 
@@ -691,11 +892,16 @@ void HandleSwitches(int argc, char *argv[])
 {
        int s;
       
-       while ((s=xgetopt(argc,argv,"bBwWnNvVGgh:H:i:I:o:O:P:p:t:T:c:C:")) != EOF) {
+       while ((s=xgetopt(argc,argv,"eEbBwWnNvVGgh:H:i:I:o:O:P:p:t:T:c:C:l:L:M:m:K:k:")) != EOF) {
 
        switch (s)
        {
 
+       case 'e':
+       case 'E':
+            EmbedProfile = TRUE;
+            break;
+
        case 'b':
        case 'B':
             BlackWhiteCompensation = TRUE;
@@ -708,13 +914,25 @@ void HandleSwitches(int argc, char *argv[])
 
        case 'i':
        case 'I':
+            if (lIsDeviceLink)
+                   FatalError("Device-link already specified"); 
+
             cInpProf = xoptarg;
             break;
 
        case 'o':
        case 'O':
+           if (lIsDeviceLink)
+                   FatalError("Device-link already specified"); 
+
            cOutProf = xoptarg;
-            break;
+           break;
+
+       case 'l':
+       case 'L': 
+                cInpProf = xoptarg;
+                lIsDeviceLink = TRUE;
+                break;
 
        case 'p':
        case 'P':
@@ -728,6 +946,14 @@ void HandleSwitches(int argc, char *argv[])
             if (Intent < 0) Intent = 0;
             break;
 
+
+       case 'm':
+       case 'M':
+            ProofingIntent = atoi(xoptarg);
+            if (ProofingIntent > 3) ProofingIntent = 3;
+            if (ProofingIntent < 0) ProofingIntent = 0;
+            break;
+
        case 'N':
        case 'n':
             IgnoreEmbedded = TRUE;
@@ -746,10 +972,18 @@ void HandleSwitches(int argc, char *argv[])
         case 'c':
         case 'C':
             PrecalcMode = atoi(xoptarg);
-            if (PrecalcMode < 0 || PrecalcMode > 2)
+            if (PrecalcMode < 0 || PrecalcMode > 3)
                     FatalError("Unknown precalc mode '%d'", PrecalcMode);
             break;
 
+        case 'k':
+        case 'K':
+                InkLimit = atof(xoptarg);
+                if (InkLimit < 0.0 || InkLimit > 400.0)
+                        FatalError("Ink limit must be 0%%..400%%");
+                break;
+                
+
         case 'H':
         case 'h':  {
 
@@ -760,7 +994,7 @@ void HandleSwitches(int argc, char *argv[])
 
   default:
 
-       FatalError("Unknown option - run without args to see valid ones.\n");
+       FatalError("Unknown option - run without args to see valid ones");
     }
        
     }
@@ -785,10 +1019,22 @@ int main(int argc, char* argv[])
               }
 
      
-      if (Verbose)
-        fprintf(stdout, "%s(%s) -> %s(%s) [%s]", argv[xoptind],
-                                                cInpProf, argv[xoptind+1],
-                                                cOutProf, Intents[Intent]);
+      if (Verbose) {
+
+        if (lIsDeviceLink)
+            fprintf(stdout, "%s(device link) -> %s [%s]", 
+                                                argv[xoptind], 
+                                                argv[xoptind+1], 
+                                                Intents[Intent]);
+
+        else
+           fprintf(stdout, "%s(%s) -> %s(%s) [%s]", argv[xoptind],
+                                                (cInpProf == NULL ? "sRGB": cInpProf), 
+                                                argv[xoptind+1],
+                                                (cOutProf == NULL ? "sRGB" : cOutProf), 
+                                                Intents[Intent]);
+        fflush(stdout);
+      }
 
       TIFFSetErrorHandler(ConsoleErrorHandler);
       TIFFSetWarningHandler(ConsoleWarningHandler);
@@ -812,7 +1058,7 @@ int main(int argc, char* argv[])
       } while (TIFFReadDirectory(in));
 
 
-      if (Verbose) fprintf(stdout, "\n");
+      if (Verbose) { fprintf(stdout, "\n"); fflush(stdout); }
 
       TIFFClose(in);
       TIFFClose(out);
@@ -820,5 +1066,3 @@ int main(int argc, char* argv[])
       return 0;
 }
 
-
-
